@@ -1,3 +1,5 @@
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set ts=8 sts=2 et sw=2 tw=80: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -8,8 +10,9 @@
 
 #include "mozilla/Assertions.h"
 #include "mozilla/dom/File.h"
-#include "mozilla/dom/ipc/BlobChild.h"
-#include "mozilla/dom/ipc/BlobParent.h"
+#include "mozilla/dom/ipc/IPCBlobInputStream.h"
+#include "mozilla/dom/ipc/IPCBlobInputStreamStorage.h"
+#include "mozilla/SlicedInputStream.h"
 #include "nsComponentManagerUtils.h"
 #include "nsDebug.h"
 #include "nsID.h"
@@ -26,20 +29,19 @@ namespace {
 
 NS_DEFINE_CID(kStringInputStreamCID, NS_STRINGINPUTSTREAM_CID);
 NS_DEFINE_CID(kFileInputStreamCID, NS_LOCALFILEINPUTSTREAM_CID);
-NS_DEFINE_CID(kPartialFileInputStreamCID, NS_PARTIALLOCALFILEINPUTSTREAM_CID);
 NS_DEFINE_CID(kBufferedInputStreamCID, NS_BUFFEREDINPUTSTREAM_CID);
 NS_DEFINE_CID(kMIMEInputStreamCID, NS_MIMEINPUTSTREAM_CID);
 NS_DEFINE_CID(kMultiplexInputStreamCID, NS_MULTIPLEXINPUTSTREAM_CID);
 
-} // anonymous namespace
+} // namespace
 
 namespace mozilla {
 namespace ipc {
 
 void
-SerializeInputStream(nsIInputStream* aInputStream,
-                     InputStreamParams& aParams,
-                     nsTArray<FileDescriptor>& aFileDescriptors)
+InputStreamHelper::SerializeInputStream(nsIInputStream* aInputStream,
+                                        InputStreamParams& aParams,
+                                        nsTArray<FileDescriptor>& aFileDescriptors)
 {
   MOZ_ASSERT(aInputStream);
 
@@ -56,28 +58,23 @@ SerializeInputStream(nsIInputStream* aInputStream,
   }
 }
 
-void
-SerializeInputStream(nsIInputStream* aInputStream,
-                     OptionalInputStreamParams& aParams,
-                     nsTArray<FileDescriptor>& aFileDescriptors)
-{
-  if (aInputStream) {
-    InputStreamParams params;
-    SerializeInputStream(aInputStream, params, aFileDescriptors);
-    aParams = params;
-  }
-  else {
-    aParams = mozilla::void_t();
-  }
-}
-
 already_AddRefed<nsIInputStream>
-DeserializeInputStream(const InputStreamParams& aParams,
-                       const nsTArray<FileDescriptor>& aFileDescriptors)
+InputStreamHelper::DeserializeInputStream(const InputStreamParams& aParams,
+                                          const nsTArray<FileDescriptor>& aFileDescriptors)
 {
   nsCOMPtr<nsIInputStream> stream;
   nsCOMPtr<nsIIPCSerializableInputStream> serializable;
 
+  // IPCBlobInputStreams are not deserializable on the parent side.
+  if (aParams.type() == InputStreamParams::TIPCBlobInputStreamParams) {
+    MOZ_ASSERT(XRE_IsParentProcess());
+    IPCBlobInputStreamStorage::Get()->GetStream(aParams.get_IPCBlobInputStreamParams().id(),
+                                                aParams.get_IPCBlobInputStreamParams().start(),
+                                                aParams.get_IPCBlobInputStreamParams().length(),
+                                                getter_AddRefs(stream));
+    return stream.forget();
+  }
+
   switch (aParams.type()) {
     case InputStreamParams::TStringInputStreamParams:
       serializable = do_CreateInstance(kStringInputStreamCID);
@@ -87,10 +84,6 @@ DeserializeInputStream(const InputStreamParams& aParams,
       serializable = do_CreateInstance(kFileInputStreamCID);
       break;
 
-    case InputStreamParams::TPartialFileInputStreamParams:
-      serializable = do_CreateInstance(kPartialFileInputStreamCID);
-      break;
-
     case InputStreamParams::TBufferedInputStreamParams:
       serializable = do_CreateInstance(kBufferedInputStreamCID);
       break;
@@ -103,41 +96,9 @@ DeserializeInputStream(const InputStreamParams& aParams,
       serializable = do_CreateInstance(kMultiplexInputStreamCID);
       break;
 
-    // When the input stream already exists in this process, all we need to do
-    // is retrieve the original instead of sending any data over the wire.
-    case InputStreamParams::TRemoteInputStreamParams: {
-      if (NS_WARN_IF(XRE_GetProcessType() != GeckoProcessType_Default)) {
-        return nullptr;
-      }
-
-      const nsID& id = aParams.get_RemoteInputStreamParams().id();
-
-      nsRefPtr<FileImpl> blobImpl = BlobParent::GetBlobImplForID(id);
-
-      MOZ_ASSERT(blobImpl, "Invalid blob contents");
-
-      // If fetching the internal stream fails, we ignore it and return a
-      // null stream.
-      nsCOMPtr<nsIInputStream> stream;
-      nsresult rv = blobImpl->GetInternalStream(getter_AddRefs(stream));
-      if (NS_FAILED(rv) || !stream) {
-        NS_WARNING("Couldn't obtain a valid stream from the blob");
-      }
-      return stream.forget();
-    }
-
-    case InputStreamParams::TSameProcessInputStreamParams: {
-      MOZ_ASSERT(aFileDescriptors.IsEmpty());
-
-      const SameProcessInputStreamParams& params =
-        aParams.get_SameProcessInputStreamParams();
-
-      stream = dont_AddRef(
-        reinterpret_cast<nsIInputStream*>(params.addRefedInputStream()));
-      MOZ_ASSERT(stream);
-
-      return stream.forget();
-    }
+    case InputStreamParams::TSlicedInputStreamParams:
+      serializable = new mozilla::SlicedInputStream();
+      break;
 
     default:
       MOZ_ASSERT(false, "Unknown params!");
@@ -157,28 +118,5 @@ DeserializeInputStream(const InputStreamParams& aParams,
   return stream.forget();
 }
 
-already_AddRefed<nsIInputStream>
-DeserializeInputStream(const OptionalInputStreamParams& aParams,
-                       const nsTArray<FileDescriptor>& aFileDescriptors)
-{
-  nsCOMPtr<nsIInputStream> stream;
-
-  switch (aParams.type()) {
-    case OptionalInputStreamParams::Tvoid_t:
-      // Leave stream null.
-      break;
-
-    case OptionalInputStreamParams::TInputStreamParams:
-      stream = DeserializeInputStream(aParams.get_InputStreamParams(),
-                                      aFileDescriptors);
-      break;
-
-    default:
-      MOZ_ASSERT(false, "Unknown params!");
-  }
-
-  return stream.forget();
-}
-
 } // namespace ipc
 } // namespace mozilla
