@@ -11,23 +11,28 @@
 #include "base/basictypes.h"
 #include "base/message_loop.h"
 
-#include "mozilla/Function.h"
+#include "nsIMemoryReporter.h"
+#include "mozilla/Atomics.h"
 #include "mozilla/DebugOnly.h"
 #include "mozilla/Monitor.h"
+#include "mozilla/MozPromise.h"
 #include "mozilla/Vector.h"
 #if defined(OS_WIN)
 #include "mozilla/ipc/Neutering.h"
 #endif // defined(OS_WIN)
 #include "mozilla/ipc/Transport.h"
-#if defined(MOZ_CRASHREPORTER) && defined(OS_WIN)
-#include "mozilla/mozalloc_oom.h"
-#include "nsExceptionHandler.h"
-#endif
 #include "MessageLink.h"
+#include "nsILabelableRunnable.h"
+#include "nsThreadUtils.h"
 
 #include <deque>
-#include <stack>
+#include <functional>
+#include <map>
 #include <math.h>
+#include <stack>
+#include <vector>
+
+class nsIEventTarget;
 
 namespace mozilla {
 namespace ipc {
@@ -61,6 +66,19 @@ enum class SyncSendError {
     ReplyError,
 };
 
+enum class ResponseRejectReason {
+    SendError,
+    ChannelClosed,
+    HandlerRejected,
+    ActorDestroyed,
+    EndGuard_,
+};
+
+template<typename T>
+using ResolveCallback = std::function<void (T&&)>;
+
+using RejectCallback = std::function<void (ResponseRejectReason)>;
+
 enum ChannelState {
     ChannelClosed,
     ChannelOpening,
@@ -72,7 +90,7 @@ enum ChannelState {
 
 class AutoEnterTransaction;
 
-class MessageChannel : HasResultCodes
+class MessageChannel : HasResultCodes, MessageLoop::DestructionObserver
 {
     friend class ProcessLink;
     friend class ThreadLink;
@@ -82,6 +100,51 @@ class MessageChannel : HasResultCodes
 
     typedef mozilla::Monitor Monitor;
 
+    // We could templatize the actor type but it would unnecessarily
+    // expand the code size. Using the actor address as the
+    // identifier is already good enough.
+    typedef void* ActorIdType;
+
+public:
+    struct UntypedCallbackHolder
+    {
+        UntypedCallbackHolder(ActorIdType aActorId,
+                              RejectCallback&& aReject)
+            : mActorId(aActorId)
+            , mReject(Move(aReject))
+        {}
+
+        virtual ~UntypedCallbackHolder() {}
+
+        void Reject(ResponseRejectReason aReason) {
+            mReject(aReason);
+        }
+
+        ActorIdType mActorId;
+        RejectCallback mReject;
+    };
+
+    template<typename Value>
+    struct CallbackHolder : public UntypedCallbackHolder
+    {
+        CallbackHolder(ActorIdType aActorId,
+                       ResolveCallback<Value>&& aResolve,
+                       RejectCallback&& aReject)
+            : UntypedCallbackHolder(aActorId, Move(aReject))
+            , mResolve(Move(aResolve))
+        {}
+
+        void Resolve(Value&& aReason) {
+            mResolve(Move(aReason));
+        }
+
+        ResolveCallback<Value> mResolve;
+    };
+
+private:
+    static Atomic<size_t> gUnresolvedResponses;
+    friend class PendingResponseReporter;
+
   public:
     static const int32_t kNoTimeout;
 
@@ -89,9 +152,14 @@ class MessageChannel : HasResultCodes
     typedef IPC::MessageInfo MessageInfo;
     typedef mozilla::ipc::Transport Transport;
 
-    explicit MessageChannel(IToplevelProtocol *aListener);
+    explicit MessageChannel(const char *aName,
+                            IToplevelProtocol *aListener);
     ~MessageChannel();
 
+    IToplevelProtocol *Listener() const {
+        return mListener;
+    }
+
     // "Open" from the perspective of the transport layer; the underlying
     // socketpair/pipe should already be created.
     //
@@ -107,7 +175,7 @@ class MessageChannel : HasResultCodes
     // For more details on the process of opening a channel between
     // threads, see the extended comment on this function
     // in MessageChannel.cpp.
-    bool Open(MessageChannel *aTargetChan, MessageLoop *aTargetLoop, Side aSide);
+    bool Open(MessageChannel *aTargetChan, nsIEventTarget *aEventTarget, Side aSide);
 
     // Close the underlying transport channel.
     void Close();
@@ -126,7 +194,7 @@ class MessageChannel : HasResultCodes
     // Call aInvoke for each pending message until it returns false.
     // XXX: You must get permission from an IPC peer to use this function
     //      since it requires custom deserialization and re-orders events.
-    void PeekMessages(mozilla::function<bool(const Message& aMsg)> aInvoke);
+    void PeekMessages(const std::function<bool(const Message& aMsg)>& aInvoke);
 
     // Misc. behavioral traits consumers can request for this channel
     enum ChannelFlags {
@@ -149,6 +217,29 @@ class MessageChannel : HasResultCodes
     // Asynchronously send a message to the other side of the channel
     bool Send(Message* aMsg);
 
+    // Asynchronously send a message to the other side of the channel
+    // and wait for asynchronous reply.
+    template<typename Value>
+    void Send(Message* aMsg,
+              ActorIdType aActorId,
+              ResolveCallback<Value>&& aResolve,
+              RejectCallback&& aReject) {
+        int32_t seqno = NextSeqno();
+        aMsg->set_seqno(seqno);
+        if (!Send(aMsg)) {
+            aReject(ResponseRejectReason::SendError);
+            return;
+        }
+
+        UniquePtr<UntypedCallbackHolder> callback =
+            MakeUnique<CallbackHolder<Value>>(
+                aActorId, Move(aResolve), Move(aReject));
+        mPendingResponses.insert(std::make_pair(seqno, Move(callback)));
+        gUnresolvedResponses++;
+    }
+
+    void SendBuildID();
+
     // Asynchronously deliver a message back to this side of the
     // channel
     bool Echo(Message* aMsg);
@@ -164,6 +255,13 @@ class MessageChannel : HasResultCodes
 
     bool CanSend() const;
 
+    // Remove and return a callback that needs reply
+    UniquePtr<UntypedCallbackHolder> PopCallback(const Message& aMsg);
+
+    // Used to reject and remove pending responses owned by the given
+    // actor when it's about to be destroyed.
+    void RejectPendingResponsesForActor(ActorIdType aActorId);
+
     // If sending a sync message returns an error, this function gives a more
     // descriptive error message.
     SyncSendError LastSendError() const {
@@ -185,6 +283,22 @@ class MessageChannel : HasResultCodes
     bool IsInTransaction() const;
     void CancelCurrentTransaction();
 
+    // Force all calls to Send to defer actually sending messages. This will
+    // cause sync messages to block until another thread calls
+    // StopPostponingSends.
+    //
+    // This must be called from the worker thread.
+    void BeginPostponingSends();
+
+    // Stop postponing sent messages, and immediately flush all postponed
+    // messages to the link. This may be called from any thread.
+    //
+    // Note that there are no ordering guarantees between two different
+    // MessageChannels. If channel B sends a message, then stops postponing
+    // channel A, messages from A may arrive before B. The easiest way to order
+    // this, if needed, is to make B send a sync message.
+    void StopPostponingSends();
+
     /**
      * This function is used by hang annotation code to determine which IPDL
      * actor is highest in the call stack at the time of the hang. It should
@@ -213,6 +327,10 @@ class MessageChannel : HasResultCodes
         sIsPumpingMessages = aIsPumping;
     }
 
+    void SetInKillHardShutdown() {
+        mInKillHardShutdown = true;
+    }
+
 #ifdef OS_WIN
     struct MOZ_STACK_CLASS SyncStackFrame
     {
@@ -347,10 +465,6 @@ class MessageChannel : HasResultCodes
     void EnteredSyncSend();
     void ExitedSyncSend();
 
-    IToplevelProtocol *Listener() const {
-        return mListener;
-    }
-
     void DebugAbort(const char* file, int line, const char* cond,
                     const char* why,
                     bool reply=false);
@@ -421,6 +535,15 @@ class MessageChannel : HasResultCodes
     // Tell the IO thread to close the channel and wait for it to ACK.
     void SynchronouslyClose();
 
+    // Returns true if ShouldDeferMessage(aMsg) is guaranteed to return true.
+    // Otherwise, the result of ShouldDeferMessage(aMsg) may be true or false,
+    // depending on context.
+    static bool IsAlwaysDeferred(const Message& aMsg);
+
+    // Helper for sending a message via the link. This should only be used for
+    // non-special messages that might have to be postponed.
+    void SendMessageToLink(Message* aMsg);
+
     bool WasTransactionCanceled(int transaction);
     bool ShouldDeferMessage(const Message& aMsg);
     bool ShouldDeferInterruptMessage(const Message& aMsg, size_t aStackDepth);
@@ -436,7 +559,8 @@ class MessageChannel : HasResultCodes
     // Can be run on either thread
     void AssertWorkerThread() const
     {
-        MOZ_RELEASE_ASSERT(mWorkerLoopID == MessageLoop::current()->id(),
+        MOZ_ASSERT(mWorkerThread, "Channel hasn't been opened yet");
+        MOZ_RELEASE_ASSERT(mWorkerThread == GetCurrentVirtualThread(),
                            "not on worker thread!");
     }
 
@@ -445,7 +569,8 @@ class MessageChannel : HasResultCodes
     // NOT our worker thread.
     void AssertLinkThread() const
     {
-        MOZ_RELEASE_ASSERT(mWorkerLoopID != MessageLoop::current()->id(),
+        MOZ_ASSERT(mWorkerThread, "Channel hasn't been opened yet");
+        MOZ_RELEASE_ASSERT(mWorkerThread != GetCurrentVirtualThread(),
                            "on worker thread but should not be!");
     }
 
@@ -453,12 +578,11 @@ class MessageChannel : HasResultCodes
     class MessageTask :
         public CancelableRunnable,
         public LinkedListElement<RefPtr<MessageTask>>,
-        public nsIRunnablePriority
+        public nsIRunnablePriority,
+        public nsILabelableRunnable
     {
     public:
-        explicit MessageTask(MessageChannel* aChannel, Message&& aMessage)
-          : mChannel(aChannel), mMessage(Move(aMessage)), mScheduled(false)
-        {}
+        explicit MessageTask(MessageChannel* aChannel, Message&& aMessage);
 
         NS_DECL_ISUPPORTS_INHERITED
 
@@ -473,6 +597,8 @@ class MessageChannel : HasResultCodes
         Message& Msg() { return mMessage; }
         const Message& Msg() const { return mMessage; }
 
+        bool GetAffectedSchedulerGroups(SchedulerGroupSet& aGroups) override;
+
     private:
         MessageTask() = delete;
         MessageTask(const MessageTask&) = delete;
@@ -488,9 +614,15 @@ class MessageChannel : HasResultCodes
 
     typedef LinkedList<RefPtr<MessageTask>> MessageQueue;
     typedef std::map<size_t, Message> MessageMap;
+    typedef std::map<size_t, UniquePtr<UntypedCallbackHolder>> CallbackMap;
     typedef IPC::Message::msgid_t msgid_t;
 
+    void WillDestroyCurrentMessageLoop() override;
+
   private:
+    // This will be a string literal, so lifetime is not an issue.
+    const char* mName;
+
     // Based on presumption the listener owns and overlives the channel,
     // this is never nullified.
     IToplevelProtocol* mListener;
@@ -501,9 +633,9 @@ class MessageChannel : HasResultCodes
     MessageLoop* mWorkerLoop;           // thread where work is done
     RefPtr<CancelableRunnable> mChannelErrorTask;  // NotifyMaybeChannelError runnable
 
-    // id() of mWorkerLoop.  This persists even after mWorkerLoop is cleared
-    // during channel shutdown.
-    int mWorkerLoopID;
+    // Thread we are allowed to send and receive on. This persists even after
+    // mWorkerLoop is cleared during channel shutdown.
+    PRThread* mWorkerThread;
 
     // Timeout periods are broken up in two to prevent system suspension from
     // triggering an abort. This method (called by WaitForEvent with a 'did
@@ -513,7 +645,7 @@ class MessageChannel : HasResultCodes
     bool mInTimeoutSecondHalf;
 
     // Worker-thread only; sequence numbers for messages that require
-    // synchronous replies.
+    // replies.
     int32_t mNextSeqno;
 
     static bool sIsPumpingMessages;
@@ -575,6 +707,12 @@ class MessageChannel : HasResultCodes
     bool DispatchingSyncMessage() const;
     int DispatchingSyncMessageNestedLevel() const;
 
+#ifdef DEBUG
+    void AssertMaybeDeferredCountCorrect();
+#else
+    void AssertMaybeDeferredCountCorrect() {}
+#endif
+
     // If a sync message times out, we store its sequence number here. Any
     // future sync messages will fail immediately. Once the reply for original
     // sync message is received, we allow sync messages again.
@@ -627,6 +765,11 @@ class MessageChannel : HasResultCodes
     //
     MessageQueue mPending;
 
+    // The number of messages in mPending for which IsAlwaysDeferred is false
+    // (i.e., the number of messages that might not be deferred, depending on
+    // context).
+    size_t mMaybeDeferredPendingCount;
+
     // Stack of all the out-calls on which this channel is awaiting responses.
     // Each stack refers to a different protocol and the stacks are mutually
     // exclusive: multiple outcalls of the same kind cannot be initiated while
@@ -682,6 +825,9 @@ class MessageChannel : HasResultCodes
     // https://bugzilla.mozilla.org/show_bug.cgi?id=521929.
     MessageMap mOutOfTurnReplies;
 
+    // Map of async Callbacks that are still waiting replies.
+    CallbackMap mPendingResponses;
+
     // Stack of Interrupt in-calls that were deferred because of race
     // conditions.
     std::stack<Message> mDeferred;
@@ -707,6 +853,13 @@ class MessageChannel : HasResultCodes
     RefPtr<CancelableRunnable> mOnChannelConnectedTask;
     bool mPeerPidSet;
     int32_t mPeerPid;
+
+    // Channels can enter messages are not sent immediately; instead, they are
+    // held in a queue until another thread deems it is safe to send them.
+    bool mIsPostponingSends;
+    std::vector<UniquePtr<Message>> mPostponedSends;
+
+    bool mInKillHardShutdown;
 };
 
 void
@@ -715,4 +868,13 @@ CancelCPOWs();
 } // namespace ipc
 } // namespace mozilla
 
+namespace IPC {
+template <>
+struct ParamTraits<mozilla::ipc::ResponseRejectReason>
+    : public ContiguousEnumSerializer<mozilla::ipc::ResponseRejectReason,
+                                      mozilla::ipc::ResponseRejectReason::SendError,
+                                      mozilla::ipc::ResponseRejectReason::EndGuard_>
+{ };
+} // namespace IPC
+
 #endif  // ifndef ipc_glue_MessageChannel_h
