@@ -18,7 +18,7 @@
 #include "nsCOMPtr.h"
 #include "nsIThreadInternal.h"
 
-class nsIThread;
+class nsIEventTarget;
 class nsITimer;
 
 namespace mozilla {
@@ -31,7 +31,7 @@ class MessagePump : public base::MessagePumpDefault
   friend class DoWorkRunnable;
 
 public:
-  explicit MessagePump(nsIThread* aThread);
+  explicit MessagePump(nsIEventTarget* aEventTarget);
 
   // From base::MessagePump.
   virtual void
@@ -60,9 +60,9 @@ private:
   void DoDelayedWork(base::MessagePump::Delegate* aDelegate);
 
 protected:
-  nsIThread* mThread;
+  nsIEventTarget* mEventTarget;
 
-  // mDelayedWorkTimer and mThread are set in Run() by this class or its
+  // mDelayedWorkTimer and mEventTarget are set in Run() by this class or its
   // subclasses.
   nsCOMPtr<nsITimer> mDelayedWorkTimer;
 
@@ -91,8 +91,8 @@ private:
 class MessagePumpForNonMainThreads final : public MessagePump
 {
 public:
-  explicit MessagePumpForNonMainThreads(nsIThread* aThread)
-    : MessagePump(aThread)
+  explicit MessagePumpForNonMainThreads(nsIEventTarget* aEventTarget)
+    : MessagePump(aEventTarget)
   { }
 
   virtual void Run(base::MessagePump::Delegate* aDelegate) override;
@@ -123,7 +123,7 @@ public:
   NS_DECL_NSITHREADOBSERVER
 
 public:
-  explicit MessagePumpForNonMainUIThreads(nsIThread* aThread) :
+  explicit MessagePumpForNonMainUIThreads(nsIEventTarget* aEventTarget) :
     mInWait(false),
     mWaitLock("mInWait")
   {
@@ -164,6 +164,43 @@ private:
 };
 #endif // defined(XP_WIN)
 
+#if defined(MOZ_WIDGET_ANDROID)
+/*`
+ * The MessagePumpForAndroidUI exists to enable IPDL in the Android UI thread. The Android
+ * UI thread event loop is controlled by Android. This prevents running an existing
+ * MessagePump implementation in the Android UI thread. In order to enable IPDL on the
+ * Android UI thread it is necessary to have a non-looping MessagePump. This class enables
+ * forwarding of nsIRunnables from MessageLoop::PostTask_Helper to the registered
+ * nsIEventTarget with out the need to control the event loop. The only member function
+ * that should be invoked is GetXPCOMThread. All other member functions will invoke MOZ_CRASH
+*/
+class MessagePumpForAndroidUI : public base::MessagePump {
+
+public:
+  MessagePumpForAndroidUI(nsIEventTarget* aEventTarget)
+    : mEventTarget(aEventTarget)
+  { }
+
+  virtual void Run(Delegate* delegate);
+  virtual void Quit();
+  virtual void ScheduleWork();
+  virtual void ScheduleDelayedWork(const base::TimeTicks& delayed_work_time);
+  virtual nsIEventTarget* GetXPCOMThread()
+  {
+    return mEventTarget;
+  }
+
+private:
+  ~MessagePumpForAndroidUI()
+  { }
+  MessagePumpForAndroidUI()
+  { }
+
+  nsIEventTarget* mEventTarget;
+};
+#endif // defined(MOZ_WIDGET_ANDROID)
+
+
 } /* namespace ipc */
 } /* namespace mozilla */
 
