@@ -12,6 +12,9 @@
 namespace js {
 namespace frontend {
 
+class FullParseHandler;
+template <class ParseHandler> class PerHandlerParser;
+
 // Perform constant folding on the given AST. For example, the program
 // `print(2 + 2)` would become `print(4)`.
 //
@@ -25,12 +28,12 @@ namespace frontend {
 //        return false;
 //    if (!FoldConstants(cx, &pn, parser))
 //        return false;
-bool
-FoldConstants(ExclusiveContext* cx, ParseNode** pnp, Parser<FullParseHandler>* parser);
+extern MOZ_MUST_USE bool
+FoldConstants(JSContext* cx, ParseNode** pnp, PerHandlerParser<FullParseHandler>* parser);
 
-inline bool
-FoldConstants(ExclusiveContext* cx, SyntaxParseHandler::Node* pnp,
-              Parser<SyntaxParseHandler>* parser)
+inline MOZ_MUST_USE bool
+FoldConstants(JSContext* cx, typename SyntaxParseHandler::Node* pnp,
+              PerHandlerParser<SyntaxParseHandler>* parser)
 {
     return true;
 }
