@@ -309,8 +309,11 @@ class Instruction {
   bool IsCBNZ() const;
   bool IsLDR() const;
   bool IsNOP() const;
+  bool IsCSDB() const;
   bool IsADR() const;
   bool IsADRP() const;
+  bool IsMovz() const;
+  bool IsMovk() const;
   bool IsBranchLinkImm() const;
   bool IsTargetReachable(Instruction* target) const;
   ptrdiff_t ImmPCRawOffset() const;
