@@ -26,7 +26,7 @@ const char OptionParser::prognameMeta[] = "{progname}";
     __cls##Option * \
     Option::as##__cls##Option() \
     { \
-        JS_ASSERT(is##__cls##Option()); \
+        MOZ_ASSERT(is##__cls##Option()); \
         return static_cast<__cls##Option*>(this); \
     } \
     const __cls##Option * \
@@ -38,7 +38,7 @@ const char OptionParser::prognameMeta[] = "{progname}";
 ValuedOption*
 Option::asValued()
 {
-    JS_ASSERT(isValued());
+    MOZ_ASSERT(isValued());
     return static_cast<ValuedOption*>(this);
 }
 
@@ -84,13 +84,18 @@ static void
 PrintParagraph(const char* text, unsigned startColno, const unsigned limitColno, bool padFirstLine)
 {
     unsigned colno = startColno;
+    unsigned indent = 0;
     const char* it = text;
 
     if (padFirstLine)
         printf("%*s", startColno, "");
 
+    /* Skip any leading spaces. */
+    while (*it != '\0' && isspace(*it))
+        ++it;
+
     while (*it != '\0') {
-        JS_ASSERT(!isspace(*it));
+        MOZ_ASSERT(!isspace(*it) || *it == '\n');
 
         /* Delimit the current token. */
         const char* limit = it;
@@ -101,11 +106,9 @@ PrintParagraph(const char* text, unsigned startColno, const unsigned limitColno,
          * If the current token is longer than the available number of columns,
          * then make a line break before printing the token.
          */
-        JS_ASSERT(limit - it > 0);
         size_t tokLen = limit - it;
-        JS_ASSERT(tokLen);
         if (tokLen + colno >= limitColno) {
-            printf("\n%*s%.*s", startColno, "", int(tokLen), it);
+            printf("\n%*s%.*s", startColno + indent, "", int(tokLen), it);
             colno = startColno + tokLen;
         } else {
             printf("%.*s", int(tokLen), it);
@@ -128,14 +131,16 @@ PrintParagraph(const char* text, unsigned startColno, const unsigned limitColno,
             colno = startColno;
             it = limit + 1;
             /* Could also have line-leading spaces. */
+            indent = 0;
             while (*it == ' ') {
                 putchar(' ');
                 ++colno;
+                ++indent;
                 ++it;
             }
             break;
           default:
-            MOZ_ASSUME_UNREACHABLE("unhandled token splitting character in text");
+            MOZ_CRASH("unhandled token splitting character in text");
         }
     }
 }
@@ -178,8 +183,8 @@ OptionParser::printHelp(const char* progname)
         putchar('\n');
     }
 
-    if (ver)
-        printf("\nVersion: %s\n\n", ver);
+    if (version)
+        printf("\nVersion: %s\n\n", version);
 
     if (!arguments.empty()) {
         printf("Arguments:\n");
@@ -187,11 +192,10 @@ OptionParser::printHelp(const char* progname)
         static const char fmt[] = "  %s ";
         size_t fmtChars = sizeof(fmt) - 2;
         size_t lhsLen = 0;
-        for (Option** it = arguments.begin(), **end = arguments.end(); it != end; ++it)
-            lhsLen = Max(lhsLen, strlen((*it)->longflag) + fmtChars);
+        for (Option* arg : arguments)
+            lhsLen = Max(lhsLen, strlen(arg->longflag) + fmtChars);
 
-        for (Option** it = arguments.begin(), **end = arguments.end(); it != end; ++it) {
-            Option* arg = *it;
+        for (Option* arg : arguments) {
             size_t chars = printf(fmt, arg->longflag);
             for (; chars < lhsLen; ++chars)
                 putchar(' ');
@@ -206,8 +210,7 @@ OptionParser::printHelp(const char* progname)
 
         /* Calculate sizes for column alignment. */
         size_t lhsLen = 0;
-        for (Option** it = options.begin(), **end = options.end(); it != end; ++it) {
-            Option* opt = *it;
+        for (Option* opt : options) {
             size_t longflagLen = strlen(opt->longflag);
 
             size_t fmtLen;
@@ -220,8 +223,7 @@ OptionParser::printHelp(const char* progname)
         }
 
         /* Print option help text. */
-        for (Option** it = options.begin(), **end = options.end(); it != end; ++it) {
-            Option* opt = *it;
+        for (Option* opt : options) {
             size_t fmtLen;
             const char* fmt = OptionFlagsToFormatInfo(opt->shortflag, opt->isValued(), &fmtLen);
             size_t chars;
@@ -243,18 +245,26 @@ OptionParser::printHelp(const char* progname)
         }
     }
 
-    return ParseHelp;
+    return EarlyExit;
+}
+
+OptionParser::Result
+OptionParser::printVersion()
+{
+    MOZ_ASSERT(version);
+    printf("%s\n", version);
+    return EarlyExit;
 }
 
 OptionParser::Result
 OptionParser::extractValue(size_t argc, char** argv, size_t* i, char** value)
 {
-    JS_ASSERT(*i < argc);
+    MOZ_ASSERT(*i < argc);
     char* eq = strchr(argv[*i], '=');
     if (eq) {
         *value = eq + 1;
         if (*value[0] == '\0')
-            return error("A value is required for option %.*s", eq - argv[*i], argv[*i]);
+            return error("A value is required for option %.*s", (int) (eq - argv[*i]), argv[*i]);
         return Okay;
     }
 
@@ -277,6 +287,8 @@ OptionParser::handleOption(Option* opt, size_t argc, char** argv, size_t* i, boo
       {
         if (opt == &helpOption)
             return printHelp(argv[0]);
+        if (opt == &versionOption)
+            return printVersion();
         opt->asBoolOption()->value = true;
         return Okay;
       }
@@ -309,7 +321,7 @@ OptionParser::handleOption(Option* opt, size_t argc, char** argv, size_t* i, boo
         return opt->asMultiStringOption()->strings.append(arg) ? Okay : Fail;
       }
       default:
-        MOZ_ASSUME_UNREACHABLE("unhandled option kind");
+        MOZ_CRASH("unhandled option kind");
     }
 }
 
@@ -336,14 +348,14 @@ OptionParser::handleArg(size_t argc, char** argv, size_t* i, bool* optionsAllowe
         return arg->asMultiStringOption()->strings.append(value) ? Okay : Fail;
       }
       default:
-        MOZ_ASSUME_UNREACHABLE("unhandled argument kind");
+        MOZ_CRASH("unhandled argument kind");
     }
 }
 
 OptionParser::Result
 OptionParser::parseArgs(int inputArgc, char** argv)
 {
-    JS_ASSERT(inputArgc >= 0);
+    MOZ_ASSERT(inputArgc >= 0);
     size_t argc = inputArgc;
     /* Permit a "no more options" capability, like |--| offers in many shell interfaces. */
     bool optionsAllowed = true;
@@ -452,20 +464,23 @@ OptionParser::getMultiStringOption(const char* longflag) const
 
 OptionParser::~OptionParser()
 {
-    for (Option** it = options.begin(), **end = options.end(); it != end; ++it)
-        js_delete<Option>(*it);
-    for (Option** it = arguments.begin(), **end = arguments.end(); it != end; ++it)
-        js_delete<Option>(*it);
+    for (Option* opt : options)
+        js_delete<Option>(opt);
+    for (Option* arg : arguments)
+        js_delete<Option>(arg);
 }
 
 Option*
 OptionParser::findOption(char shortflag)
 {
-    for (Option** it = options.begin(), **end = options.end(); it != end; ++it) {
-        if ((*it)->shortflag == shortflag)
-            return *it;
+    for (Option* opt : options) {
+        if (opt->shortflag == shortflag)
+            return opt;
     }
 
+    if (versionOption.shortflag == shortflag)
+        return &versionOption;
+
     return helpOption.shortflag == shortflag ? &helpOption : nullptr;
 }
 
@@ -478,9 +493,9 @@ OptionParser::findOption(char shortflag) const
 Option*
 OptionParser::findOption(const char* longflag)
 {
-    for (Option** it = options.begin(), **end = options.end(); it != end; ++it) {
-        const char* target = (*it)->longflag;
-        if ((*it)->isValued()) {
+    for (Option* opt : options) {
+        const char* target = opt->longflag;
+        if (opt->isValued()) {
             size_t targetLen = strlen(target);
             /* Permit a trailing equals sign on the longflag argument. */
             for (size_t i = 0; i < targetLen; ++i) {
@@ -488,14 +503,17 @@ OptionParser::findOption(const char* longflag)
                     goto no_match;
             }
             if (longflag[targetLen] == '\0' || longflag[targetLen] == '=')
-                return *it;
+                return opt;
         } else {
             if (strcmp(target, longflag) == 0)
-                return *it;
+                return opt;
         }
   no_match:;
     }
 
+    if (strcmp(versionOption.longflag, longflag) == 0)
+        return &versionOption;
+
     return strcmp(helpOption.longflag, longflag) ? nullptr : &helpOption;
 }
 
@@ -615,7 +633,7 @@ OptionParser::addOptionalStringArg(const char* name, const char* help)
 bool
 OptionParser::addOptionalMultiStringArg(const char* name, const char* help)
 {
-    JS_ASSERT_IF(!arguments.empty(), !arguments.back()->isVariadic());
+    MOZ_ASSERT_IF(!arguments.empty(), !arguments.back()->isVariadic());
     if (!arguments.reserve(arguments.length() + 1))
         return false;
     MultiStringOption* mso = js_new<MultiStringOption>(1, name, help, (const char*) nullptr);
