@@ -9,19 +9,18 @@
 
 #include "vm/BooleanObject.h"
 
-#include "jsobjinlines.h"
+#include "vm/JSObject-inl.h"
 
 namespace js {
 
 inline BooleanObject*
-BooleanObject::create(JSContext* cx, bool b)
+BooleanObject::create(JSContext* cx, bool b, HandleObject proto /* = nullptr */)
 {
-    JSObject* obj = NewBuiltinClassInstance(cx, &class_);
+    BooleanObject* obj = NewObjectWithClassProto<BooleanObject>(cx, proto);
     if (!obj)
         return nullptr;
-    BooleanObject& boolobj = obj->as<BooleanObject>();
-    boolobj.setPrimitiveValue(b);
-    return &boolobj;
+    obj->setPrimitiveValue(b);
+    return obj;
 }
 
 } // namespace js
