@@ -9,7 +9,7 @@
 #include "AccessCheck.h"
 #include "xpcprivate.h"
 #include "jsapi.h"
-#include "jswrapper.h"
+#include "js/Wrapper.h"
 #include "nsXULAppAPI.h"
 
 using namespace JS;
@@ -21,21 +21,21 @@ const ChromeObjectWrapper ChromeObjectWrapper::singleton;
 bool
 ChromeObjectWrapper::defineProperty(JSContext* cx, HandleObject wrapper,
                                     HandleId id,
-                                    MutableHandle<JSPropertyDescriptor> desc) const
+                                    Handle<PropertyDescriptor> desc,
+                                    ObjectOpResult& result) const
 {
     if (!AccessCheck::checkPassToPrivilegedCode(cx, wrapper, desc.value()))
         return false;
-    return ChromeObjectWrapperBase::defineProperty(cx, wrapper, id, desc);
+    return ChromeObjectWrapperBase::defineProperty(cx, wrapper, id, desc, result);
 }
 
 bool
-ChromeObjectWrapper::set(JSContext* cx, HandleObject wrapper,
-                         HandleObject receiver, HandleId id,
-                         bool strict, MutableHandleValue vp) const
+ChromeObjectWrapper::set(JSContext* cx, HandleObject wrapper, HandleId id, HandleValue v,
+                         HandleValue receiver, ObjectOpResult& result) const
 {
-    if (!AccessCheck::checkPassToPrivilegedCode(cx, wrapper, vp))
+    if (!AccessCheck::checkPassToPrivilegedCode(cx, wrapper, v))
         return false;
-    return ChromeObjectWrapperBase::set(cx, wrapper, receiver, id, strict, vp);
+    return ChromeObjectWrapperBase::set(cx, wrapper, id, v, receiver, result);
 }
 
-}
+} // namespace xpc
