@@ -8,6 +8,8 @@
 #include "AccessCheck.h"
 #include "ChromeObjectWrapper.h"
 #include "XrayWrapper.h"
+#include "nsJSUtils.h"
+#include "mozilla/ErrorResult.h"
 
 #include "jsapi.h"
 
@@ -16,6 +18,36 @@ using namespace js;
 
 namespace xpc {
 
+static JS::SymbolCode sCrossOriginWhitelistedSymbolCodes[] = {
+    JS::SymbolCode::toStringTag,
+    JS::SymbolCode::hasInstance,
+    JS::SymbolCode::isConcatSpreadable
+};
+
+static bool
+IsCrossOriginWhitelistedSymbol(JSContext* cx, JS::HandleId id)
+{
+    if (!JSID_IS_SYMBOL(id)) {
+        return false;
+    }
+
+    JS::Symbol* symbol = JSID_TO_SYMBOL(id);
+    for (auto code : sCrossOriginWhitelistedSymbolCodes) {
+        if (symbol == JS::GetWellKnownSymbol(cx, code)) {
+            return true;
+        }
+    }
+
+    return false;
+}
+
+bool
+IsCrossOriginWhitelistedProp(JSContext* cx, JS::HandleId id)
+{
+    return id == GetJSIDByIndex(cx, XPCJSContext::IDX_THEN) ||
+           IsCrossOriginWhitelistedSymbol(cx, id);
+}
+
 template <typename Policy>
 static bool
 Filter(JSContext* cx, HandleObject wrapper, AutoIdVector& props)
@@ -29,13 +61,15 @@ Filter(JSContext* cx, HandleObject wrapper, AutoIdVector& props)
         else if (JS_IsExceptionPending(cx))
             return false;
     }
-    props.resize(w);
+    if (!props.resize(w))
+        return false;
+
     return true;
 }
 
 template <typename Policy>
 static bool
-FilterPropertyDescriptor(JSContext* cx, HandleObject wrapper, HandleId id, JS::MutableHandle<JSPropertyDescriptor> desc)
+FilterPropertyDescriptor(JSContext* cx, HandleObject wrapper, HandleId id, MutableHandle<PropertyDescriptor> desc)
 {
     MOZ_ASSERT(!JS_IsExceptionPending(cx));
     bool getAllowed = Policy::check(cx, wrapper, id, Wrapper::GET);
@@ -68,7 +102,7 @@ template <typename Base, typename Policy>
 bool
 FilteringWrapper<Base, Policy>::getPropertyDescriptor(JSContext* cx, HandleObject wrapper,
                                                       HandleId id,
-                                                      JS::MutableHandle<JSPropertyDescriptor> desc) const
+                                                      MutableHandle<PropertyDescriptor> desc) const
 {
     assertEnteredPolicy(cx, wrapper, id, BaseProxyHandler::GET | BaseProxyHandler::SET |
                                          BaseProxyHandler::GET_PROPERTY_DESCRIPTOR);
@@ -81,7 +115,7 @@ template <typename Base, typename Policy>
 bool
 FilteringWrapper<Base, Policy>::getOwnPropertyDescriptor(JSContext* cx, HandleObject wrapper,
                                                          HandleId id,
-                                                         JS::MutableHandle<JSPropertyDescriptor> desc) const
+                                                         MutableHandle<PropertyDescriptor> desc) const
 {
     assertEnteredPolicy(cx, wrapper, id, BaseProxyHandler::GET | BaseProxyHandler::SET |
                                          BaseProxyHandler::GET_PROPERTY_DESCRIPTOR);
@@ -112,16 +146,15 @@ FilteringWrapper<Base, Policy>::getOwnEnumerablePropertyKeys(JSContext* cx,
 }
 
 template <typename Base, typename Policy>
-bool
-FilteringWrapper<Base, Policy>::enumerate(JSContext* cx, HandleObject wrapper,
-                                          MutableHandleObject objp) const
+JSObject*
+FilteringWrapper<Base, Policy>::enumerate(JSContext* cx, HandleObject wrapper) const
 {
     assertEnteredPolicy(cx, wrapper, JSID_VOID, BaseProxyHandler::ENUMERATE);
     // We refuse to trigger the enumerate hook across chrome wrappers because
     // we don't know how to censor custom iterator objects. Instead we trigger
     // the default proxy enumerate trap, which will use js::GetPropertyKeys
     // for the list of (censored) ids.
-    return js::BaseProxyHandler::enumerate(cx, wrapper, objp);
+    return js::BaseProxyHandler::enumerate(cx, wrapper);
 }
 
 template <typename Base, typename Policy>
@@ -147,7 +180,7 @@ FilteringWrapper<Base, Policy>::construct(JSContext* cx, JS::Handle<JSObject*> w
 template <typename Base, typename Policy>
 bool
 FilteringWrapper<Base, Policy>::nativeCall(JSContext* cx, JS::IsAcceptableThis test,
-                                           JS::NativeImpl impl, JS::CallArgs args) const
+                                           JS::NativeImpl impl, const JS::CallArgs& args) const
 {
     if (Policy::allowNativeCall(cx, test, impl))
         return Base::Permissive::nativeCall(cx, test, impl, args);
@@ -156,16 +189,8 @@ FilteringWrapper<Base, Policy>::nativeCall(JSContext* cx, JS::IsAcceptableThis t
 
 template <typename Base, typename Policy>
 bool
-FilteringWrapper<Base, Policy>::defaultValue(JSContext* cx, HandleObject obj,
-                                             JSType hint, MutableHandleValue vp) const
-{
-    return Base::defaultValue(cx, obj, hint, vp);
-}
-
-template <typename Base, typename Policy>
-bool
-FilteringWrapper<Base, Policy>::getPrototypeOf(JSContext* cx, JS::HandleObject wrapper,
-                                               JS::MutableHandleObject protop) const
+FilteringWrapper<Base, Policy>::getPrototype(JSContext* cx, JS::HandleObject wrapper,
+                                             JS::MutableHandleObject protop) const
 {
     // Filtering wrappers do not allow access to the prototype.
     protop.set(nullptr);
@@ -175,39 +200,55 @@ FilteringWrapper<Base, Policy>::getPrototypeOf(JSContext* cx, JS::HandleObject w
 template <typename Base, typename Policy>
 bool
 FilteringWrapper<Base, Policy>::enter(JSContext* cx, HandleObject wrapper,
-                                      HandleId id, Wrapper::Action act, bool* bp) const
+                                      HandleId id, Wrapper::Action act,
+                                      bool mayThrow, bool* bp) const
 {
     if (!Policy::check(cx, wrapper, id, act)) {
-        *bp = JS_IsExceptionPending(cx) ? false : Policy::deny(act, id);
+        *bp = JS_IsExceptionPending(cx) ?
+            false : Policy::deny(cx, act, id, mayThrow);
         return false;
     }
     *bp = true;
     return true;
 }
 
-CrossOriginXrayWrapper::CrossOriginXrayWrapper(unsigned flags) : SecurityXrayDOM(flags)
-{
-}
-
 bool
 CrossOriginXrayWrapper::getPropertyDescriptor(JSContext* cx,
                                               JS::Handle<JSObject*> wrapper,
                                               JS::Handle<jsid> id,
-                                              JS::MutableHandle<JSPropertyDescriptor> desc) const
+                                              JS::MutableHandle<PropertyDescriptor> desc) const
 {
     if (!SecurityXrayDOM::getPropertyDescriptor(cx, wrapper, id, desc))
         return false;
     if (desc.object()) {
+        // Cross-origin DOM objects do not have symbol-named properties apart
+        // from the ones we add ourselves here.
+        MOZ_ASSERT(!JSID_IS_SYMBOL(id),
+                   "What's this symbol-named property that appeared on a "
+                   "Window or Location instance?");
+
         // All properties on cross-origin DOM objects are |own|.
         desc.object().set(wrapper);
 
-        // All properties on cross-origin DOM objects are non-enumerable and
-        // "configurable". Any value attributes are read-only.
-        desc.attributesRef() &= ~JSPROP_ENUMERATE;
+        // All properties on cross-origin DOM objects are "configurable". Any
+        // value attributes are read-only.  Indexed properties are enumerable,
+        // but nothing else is.
+        if (!JSID_IS_INT(id)) {
+            desc.attributesRef() &= ~JSPROP_ENUMERATE;
+        }
         desc.attributesRef() &= ~JSPROP_PERMANENT;
         if (!desc.getter() && !desc.setter())
             desc.attributesRef() |= JSPROP_READONLY;
+    } else if (IsCrossOriginWhitelistedProp(cx, id)) {
+        // Spec says to return PropertyDescriptor {
+        //   [[Value]]: undefined, [[Writable]]: false, [[Enumerable]]: false,
+        //   [[Configurable]]: true
+        // }.
+        //
+        desc.setDataDescriptor(JS::UndefinedHandleValue, JSPROP_READONLY);
+        desc.object().set(wrapper);
     }
+
     return true;
 }
 
@@ -215,7 +256,7 @@ bool
 CrossOriginXrayWrapper::getOwnPropertyDescriptor(JSContext* cx,
                                                  JS::Handle<JSObject*> wrapper,
                                                  JS::Handle<jsid> id,
-                                                 JS::MutableHandle<JSPropertyDescriptor> desc) const
+                                                 JS::MutableHandle<PropertyDescriptor> desc) const
 {
     // All properties on cross-origin DOM objects are |own|.
     return getPropertyDescriptor(cx, wrapper, id, desc);
@@ -228,26 +269,79 @@ CrossOriginXrayWrapper::ownPropertyKeys(JSContext* cx, JS::Handle<JSObject*> wra
     // All properties on cross-origin objects are supposed |own|, despite what
     // the underlying native object may report. Override the inherited trap to
     // avoid passing JSITER_OWNONLY as a flag.
-    return SecurityXrayDOM::getPropertyKeys(cx, wrapper, JSITER_HIDDEN, props);
+    if (!SecurityXrayDOM::getPropertyKeys(cx, wrapper, JSITER_HIDDEN, props)) {
+        return false;
+    }
+
+    // Add "then" if it's not already in the list.
+    AutoIdVector thenProp(cx);
+    if (!thenProp.append(GetJSIDByIndex(cx, XPCJSContext::IDX_THEN))) {
+        return false;
+    }
+
+    if (!AppendUnique(cx, props, thenProp)) {
+        return false;
+    }
+
+    // Now add the three symbol-named props cross-origin objects have.
+#ifdef DEBUG
+    for (size_t n = 0; n < props.length(); ++n) {
+        MOZ_ASSERT(!JSID_IS_SYMBOL(props[n]),
+                   "Unexpected existing symbol-name prop");
+    }
+#endif
+    if (!props.reserve(props.length() +
+                       ArrayLength(sCrossOriginWhitelistedSymbolCodes))) {
+        return false;
+    }
+
+    for (auto code : sCrossOriginWhitelistedSymbolCodes) {
+        props.infallibleAppend(SYMBOL_TO_JSID(JS::GetWellKnownSymbol(cx, code)));
+    }
+
+    return true;
 }
 
 bool
 CrossOriginXrayWrapper::defineProperty(JSContext* cx, JS::Handle<JSObject*> wrapper,
                                        JS::Handle<jsid> id,
-                                       JS::MutableHandle<JSPropertyDescriptor> desc) const
+                                       JS::Handle<PropertyDescriptor> desc,
+                                       JS::ObjectOpResult& result) const
 {
-    JS_ReportError(cx, "Permission denied to define property on cross-origin object");
+    AccessCheck::reportCrossOriginDenial(cx, id, NS_LITERAL_CSTRING("define"));
     return false;
 }
 
 bool
 CrossOriginXrayWrapper::delete_(JSContext* cx, JS::Handle<JSObject*> wrapper,
-                                JS::Handle<jsid> id, bool* bp) const
+                                JS::Handle<jsid> id, JS::ObjectOpResult& result) const
 {
-    JS_ReportError(cx, "Permission denied to delete property on cross-origin object");
+    AccessCheck::reportCrossOriginDenial(cx, id, NS_LITERAL_CSTRING("delete"));
     return false;
 }
 
+bool
+CrossOriginXrayWrapper::setPrototype(JSContext* cx, JS::HandleObject wrapper,
+                                     JS::HandleObject proto,
+                                     JS::ObjectOpResult& result) const
+{
+    // https://html.spec.whatwg.org/multipage/browsers.html#windowproxy-setprototypeof
+    // and
+    // https://html.spec.whatwg.org/multipage/browsers.html#location-setprototypeof
+    // both say to call SetImmutablePrototype, which does nothing and just
+    // returns whether the passed-in value equals the current prototype.  Our
+    // current prototype is always null, so this just comes down to returning
+    // whether null was passed in.
+    //
+    // In terms of ObjectOpResult that means calling one of the fail*() things
+    // on it if non-null was passed, and it's got one that does just what we
+    // want.
+    if (!proto) {
+        return result.succeed();
+    }
+    return result.failCantSetProto();
+}
+
 #define XOW FilteringWrapper<CrossOriginXrayWrapper, CrossOriginAccessiblePropertiesOnly>
 #define NNXOW FilteringWrapper<CrossCompartmentSecurityWrapper, Opaque>
 #define NNXOWC FilteringWrapper<CrossCompartmentSecurityWrapper, OpaqueWithCall>
@@ -260,4 +354,4 @@ template class XOW;
 template class NNXOW;
 template class NNXOWC;
 template class ChromeObjectWrapperBase;
-}
+} // namespace xpc
