@@ -9,38 +9,31 @@
 
 #include "XrayWrapper.h"
 #include "mozilla/Attributes.h"
-#include "jswrapper.h"
 #include "js/CallNonGenericMethod.h"
-
-struct JSPropertyDescriptor;
-
-namespace JS {
-class AutoIdVector;
-}
+#include "js/Wrapper.h"
 
 namespace xpc {
 
 template <typename Base, typename Policy>
 class FilteringWrapper : public Base {
   public:
-    MOZ_CONSTEXPR explicit FilteringWrapper(unsigned flags) : Base(flags) {}
+    constexpr explicit FilteringWrapper(unsigned flags) : Base(flags) {}
 
     virtual bool enter(JSContext* cx, JS::Handle<JSObject*> wrapper, JS::Handle<jsid> id,
-                       js::Wrapper::Action act, bool* bp) const override;
+                       js::Wrapper::Action act, bool mayThrow, bool* bp) const override;
 
     virtual bool getOwnPropertyDescriptor(JSContext* cx, JS::Handle<JSObject*> wrapper,
                                           JS::Handle<jsid> id,
-                                          JS::MutableHandle<JSPropertyDescriptor> desc) const override;
+                                          JS::MutableHandle<JS::PropertyDescriptor> desc) const override;
     virtual bool ownPropertyKeys(JSContext* cx, JS::Handle<JSObject*> wrapper,
                                  JS::AutoIdVector& props) const override;
 
     virtual bool getPropertyDescriptor(JSContext* cx, JS::Handle<JSObject*> wrapper,
                                        JS::Handle<jsid> id,
-                                       JS::MutableHandle<JSPropertyDescriptor> desc) const override;
+                                       JS::MutableHandle<JS::PropertyDescriptor> desc) const override;
     virtual bool getOwnEnumerablePropertyKeys(JSContext* cx, JS::Handle<JSObject*> wrapper,
                                               JS::AutoIdVector& props) const override;
-    virtual bool enumerate(JSContext* cx, JS::Handle<JSObject*> wrapper,
-                           JS::MutableHandle<JSObject*> objp) const override;
+    virtual JSObject* enumerate(JSContext* cx, JS::Handle<JSObject*> wrapper) const override;
 
     virtual bool call(JSContext* cx, JS::Handle<JSObject*> wrapper,
                       const JS::CallArgs& args) const override;
@@ -48,13 +41,10 @@ class FilteringWrapper : public Base {
                            const JS::CallArgs& args) const override;
 
     virtual bool nativeCall(JSContext* cx, JS::IsAcceptableThis test, JS::NativeImpl impl,
-                            JS::CallArgs args) const override;
+                            const JS::CallArgs& args) const override;
 
-    virtual bool defaultValue(JSContext* cx, JS::Handle<JSObject*> obj, JSType hint,
-                              JS::MutableHandleValue vp) const override;
-
-    virtual bool getPrototypeOf(JSContext* cx, JS::HandleObject wrapper,
-                                JS::MutableHandleObject protop) const override;
+    virtual bool getPrototype(JSContext* cx, JS::HandleObject wrapper,
+                              JS::MutableHandleObject protop) const override;
 
     static const FilteringWrapper singleton;
 };
@@ -66,24 +56,36 @@ class FilteringWrapper : public Base {
  */
 class CrossOriginXrayWrapper : public SecurityXrayDOM {
   public:
-    explicit CrossOriginXrayWrapper(unsigned flags);
+    constexpr explicit CrossOriginXrayWrapper(unsigned flags) :
+      SecurityXrayDOM(flags) {}
+
 
     virtual bool getOwnPropertyDescriptor(JSContext* cx, JS::Handle<JSObject*> wrapper,
                                           JS::Handle<jsid> id,
-                                          JS::MutableHandle<JSPropertyDescriptor> desc) const override;
+                                          JS::MutableHandle<JS::PropertyDescriptor> desc) const override;
     virtual bool defineProperty(JSContext* cx, JS::Handle<JSObject*> wrapper,
                                 JS::Handle<jsid> id,
-                                JS::MutableHandle<JSPropertyDescriptor> desc) const override;
+                                JS::Handle<JS::PropertyDescriptor> desc,
+                                JS::ObjectOpResult& result) const override;
     virtual bool ownPropertyKeys(JSContext* cx, JS::Handle<JSObject*> wrapper,
                                  JS::AutoIdVector& props) const override;
     virtual bool delete_(JSContext* cx, JS::Handle<JSObject*> wrapper,
-                         JS::Handle<jsid> id, bool* bp) const override;
+                         JS::Handle<jsid> id, JS::ObjectOpResult& result) const override;
 
     virtual bool getPropertyDescriptor(JSContext* cx, JS::Handle<JSObject*> wrapper,
                                        JS::Handle<jsid> id,
-                                       JS::MutableHandle<JSPropertyDescriptor> desc) const override;
+                                       JS::MutableHandle<JS::PropertyDescriptor> desc) const override;
+
+    virtual bool setPrototype(JSContext* cx, JS::HandleObject wrapper,
+                              JS::HandleObject proto,
+                              JS::ObjectOpResult& result) const override;
 };
 
-}
+// Check whether the given jsid is a property name (string or symbol) whose value
+// can be gotten cross-origin.  Cross-origin gets always return undefined as the
+// value, unless the Xray actually provides a different value.
+bool IsCrossOriginWhitelistedProp(JSContext* cx, JS::HandleId id);
+
+} // namespace xpc
 
 #endif /* __FilteringWrapper_h__ */
