@@ -1,3 +1,5 @@
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set ts=8 sts=2 et sw=2 tw=80: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -27,6 +29,6 @@ FindFrameTargetedByInputEvent(WidgetGUIEvent* aEvent,
                               const nsPoint& aPointRelativeToRootFrame,
                               uint32_t aFlags = 0);
 
-}
+} // namespace mozilla
 
 #endif /* mozilla_PositionedEventTargeting_h */
