@@ -1,8 +1,8 @@
-/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* vim: set ts=2 et sw=2 tw=80: */
-/* This Source Code is subject to the terms of the Mozilla Public License
- * version 2.0 (the "License"). You can obtain a copy of the License at
- * http://mozilla.org/MPL/2.0/. */
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set ts=8 sts=2 et sw=2 tw=80: */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 /* base class for ruby rendering objects that directly contain content */
 
@@ -15,13 +15,6 @@ using namespace mozilla;
 
 //----------------------------------------------------------------------
 
-// Frame class boilerplate
-// =======================
-
-NS_IMPL_FRAMEARENA_HELPERS(nsRubyContentFrame)
-
-//----------------------------------------------------------------------
-
 // nsRubyContentFrame Method Implementations
 // ======================================
 
@@ -31,13 +24,13 @@ nsRubyContentFrame::IsFrameOfType(uint32_t aFlags) const
   if (aFlags & eBidiInlineContainer) {
     return false;
   }
-  return nsRubyContentFrameSuper::IsFrameOfType(aFlags);
+  return nsInlineFrame::IsFrameOfType(aFlags);
 }
 
 bool
 nsRubyContentFrame::IsIntraLevelWhitespace() const
 {
-  nsIAtom* pseudoType = StyleContext()->GetPseudo();
+  nsAtom* pseudoType = StyleContext()->GetPseudo();
   if (pseudoType != nsCSSAnonBoxes::rubyBase &&
       pseudoType != nsCSSAnonBoxes::rubyText) {
     return false;
