@@ -1,8 +1,8 @@
-/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
-/* vim: set ts=2 et sw=2 tw=80: */
-/* This Source Code is subject to the terms of the Mozilla Public License
- * version 2.0 (the "License"). You can obtain a copy of the License at
- * http://mozilla.org/MPL/2.0/. */
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set ts=8 sts=2 et sw=2 tw=80: */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 /* base class for ruby rendering objects that directly contain content */
 
@@ -11,12 +11,10 @@
 
 #include "nsInlineFrame.h"
 
-typedef nsInlineFrame nsRubyContentFrameSuper;
-
-class nsRubyContentFrame : public nsRubyContentFrameSuper
+class nsRubyContentFrame : public nsInlineFrame
 {
 public:
-  NS_DECL_FRAMEARENA_HELPERS
+  NS_DECL_ABSTRACT_FRAME(nsRubyContentFrame)
 
   // nsIFrame overrides
   virtual bool IsFrameOfType(uint32_t aFlags) const override;
@@ -29,8 +27,9 @@ public:
   bool IsIntraLevelWhitespace() const;
 
 protected:
-  explicit nsRubyContentFrame(nsStyleContext* aContext)
-    : nsRubyContentFrameSuper(aContext) {}
+  nsRubyContentFrame(nsStyleContext* aContext, ClassID aID)
+    : nsInlineFrame(aContext, aID)
+  {}
 };
 
 #endif /* nsRubyContentFrame_h___ */
