@@ -1,5 +1,6 @@
-/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
- * This Source Code Form is subject to the terms of the Mozilla Public
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set ts=8 sts=2 et sw=2 tw=80: */
+/* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
@@ -27,7 +28,7 @@ struct nsTransformedCharStyle final {
     , mExplicitLanguage(aContext->StyleFont()->mExplicitLanguage) {}
 
   nsFont                  mFont;
-  nsCOMPtr<nsIAtom>       mLanguage;
+  RefPtr<nsAtom>       mLanguage;
   RefPtr<nsPresContext> mPresContext;
   float                   mScriptSizeMultiplier;
   uint8_t                 mTextTransform;
@@ -49,14 +50,18 @@ public:
   already_AddRefed<nsTransformedTextRun>
   MakeTextRun(const uint8_t* aString, uint32_t aLength,
               const gfxFontGroup::Parameters* aParams,
-              gfxFontGroup* aFontGroup, uint32_t aFlags,
+              gfxFontGroup* aFontGroup,
+              mozilla::gfx::ShapedTextFlags aFlags,
+              nsTextFrameUtils::Flags aFlags2,
               nsTArray<RefPtr<nsTransformedCharStyle>>&& aStyles,
               bool aOwnsFactory);
 
   already_AddRefed<nsTransformedTextRun>
   MakeTextRun(const char16_t* aString, uint32_t aLength,
               const gfxFontGroup::Parameters* aParams,
-              gfxFontGroup* aFontGroup, uint32_t aFlags,
+              gfxFontGroup* aFontGroup,
+              mozilla::gfx::ShapedTextFlags aFlags,
+              nsTextFrameUtils::Flags aFlags2,
               nsTArray<RefPtr<nsTransformedCharStyle>>&& aStyles,
               bool aOwnsFactory);
 
@@ -76,9 +81,9 @@ public:
   // MakeTextRun (after making it virtual in the superclass) and have it
   // just convert the string to uppercase or lowercase and create the textrun
   // via the fontgroup.
-  
+
   // Takes ownership of aInnerTransformTextRunFactory
-  explicit nsCaseTransformTextRunFactory(UniquePtr<nsTransformingTextRunFactory> aInnerTransformingTextRunFactory,
+  explicit nsCaseTransformTextRunFactory(mozilla::UniquePtr<nsTransformingTextRunFactory> aInnerTransformingTextRunFactory,
                                          bool aAllUppercase = false)
     : mInnerTransformingTextRunFactory(Move(aInnerTransformingTextRunFactory)),
       mAllUppercase(aAllUppercase) {}
@@ -100,7 +105,7 @@ public:
   static bool TransformString(const nsAString& aString,
                               nsString& aConvertedString,
                               bool aAllUppercase,
-                              const nsIAtom* aLanguage,
+                              const nsAtom* aLanguage,
                               nsTArray<bool>& aCharsToMergeArray,
                               nsTArray<bool>& aDeletedCharsArray,
                               const nsTransformedTextRun* aTextRun = nullptr,
@@ -125,7 +130,8 @@ public:
          nsTransformingTextRunFactory* aFactory,
          gfxFontGroup* aFontGroup,
          const char16_t* aString, uint32_t aLength,
-         const uint32_t aFlags,
+         const mozilla::gfx::ShapedTextFlags aFlags,
+         const nsTextFrameUtils::Flags aFlags2,
          nsTArray<RefPtr<nsTransformedCharStyle>>&& aStyles,
          bool aOwnsFactory);
 
@@ -134,11 +140,11 @@ public:
       delete mFactory;
     }
   }
-  
+
   void SetCapitalization(uint32_t aStart, uint32_t aLength,
                          bool* aCapitalization);
   virtual bool SetPotentialLineBreaks(Range aRange,
-                                      const uint8_t* aBreakBefore);
+                                      const uint8_t* aBreakBefore) override;
   /**
    * Called after SetCapitalization and SetPotentialLineBreaks
    * are done and before we request any data from the textrun. Also always
@@ -154,8 +160,8 @@ public:
   }
 
   // override the gfxTextRun impls to account for additional members here
-  virtual size_t SizeOfExcludingThis(mozilla::MallocSizeOf aMallocSizeOf) MOZ_MUST_OVERRIDE;
-  virtual size_t SizeOfIncludingThis(mozilla::MallocSizeOf aMallocSizeOf) MOZ_MUST_OVERRIDE;
+  virtual size_t SizeOfExcludingThis(mozilla::MallocSizeOf aMallocSizeOf) override;
+  virtual size_t SizeOfIncludingThis(mozilla::MallocSizeOf aMallocSizeOf) override;
 
   nsTransformingTextRunFactory       *mFactory;
   nsTArray<RefPtr<nsTransformedCharStyle>> mStyles;
@@ -169,10 +175,11 @@ private:
                        nsTransformingTextRunFactory* aFactory,
                        gfxFontGroup* aFontGroup,
                        const char16_t* aString, uint32_t aLength,
-                       const uint32_t aFlags,
+                       const mozilla::gfx::ShapedTextFlags aFlags,
+                       const nsTextFrameUtils::Flags aFlags2,
                        nsTArray<RefPtr<nsTransformedCharStyle>>&& aStyles,
                        bool aOwnsFactory)
-    : gfxTextRun(aParams, aLength, aFontGroup, aFlags),
+    : gfxTextRun(aParams, aLength, aFontGroup, aFlags, aFlags2),
       mFactory(aFactory), mStyles(aStyles), mString(aString, aLength),
       mOwnsFactory(aOwnsFactory), mNeedsRebuild(true)
   {
@@ -215,7 +222,8 @@ MergeCharactersInTextRun(gfxTextRun* aDest, gfxTextRun* aSrc,
                          const bool* aCharsToMerge, const bool* aDeletedChars);
 
 gfxTextRunFactory::Parameters
-GetParametersForInner(nsTransformedTextRun* aTextRun, uint32_t* aFlags,
+GetParametersForInner(nsTransformedTextRun* aTextRun,
+                      mozilla::gfx::ShapedTextFlags* aFlags,
                       mozilla::gfx::DrawTarget* aRefDrawTarget);
 
 
