@@ -1,4 +1,5 @@
-/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set ts=8 sts=2 et sw=2 tw=80: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -6,21 +7,20 @@
 #ifndef nsMathMLChar_h___
 #define nsMathMLChar_h___
 
-#include "nsAutoPtr.h"
 #include "nsColor.h"
 #include "nsMathMLOperators.h"
 #include "nsPoint.h"
 #include "nsRect.h"
 #include "nsString.h"
 #include "nsBoundingMetrics.h"
-#include "gfxFont.h"
+#include "gfxTextRun.h"
 
+class gfxContext;
 class nsGlyphTable;
 class nsIFrame;
 class nsDisplayListBuilder;
 class nsDisplayListSet;
 class nsPresContext;
-class nsRenderingContext;
 struct nsBoundingMetrics;
 class nsStyleContext;
 struct nsFont;
@@ -61,7 +61,7 @@ struct nsGlyphCode {
   bool IsGlyphID() const { return font == -1; }
 
   int32_t Length() const {
-    return (IsGlyphID() || code[1] == PRUnichar('\0') ? 1 : 2);
+    return (IsGlyphID() || code[1] == char16_t('\0') ? 1 : 2);
   }
   bool Exists() const
   {
@@ -85,6 +85,9 @@ struct nsGlyphCode {
 class nsMathMLChar
 {
 public:
+  typedef gfxTextRun::Range Range;
+  typedef mozilla::gfx::DrawTarget DrawTarget;
+
   // constructor and destructor
   nsMathMLChar() {
     MOZ_COUNT_CTOR(nsMathMLChar);
@@ -103,9 +106,9 @@ public:
                const nsDisplayListSet& aLists,
                uint32_t                aIndex,
                const nsRect*           aSelectedRect = nullptr);
-          
-  void PaintForeground(nsPresContext* aPresContext,
-                       nsRenderingContext& aRenderingContext,
+
+  void PaintForeground(nsIFrame* aForFrame,
+                       gfxContext& aRenderingContext,
                        nsPoint aPt,
                        bool aIsSelected);
 
@@ -113,8 +116,8 @@ public:
   // @param aContainerSize - IN - suggested size for the stretched char
   // @param aDesiredStretchSize - OUT - the size that the char wants
   nsresult
-  Stretch(nsPresContext*           aPresContext,
-          nsRenderingContext&     aRenderingContext,
+  Stretch(nsIFrame*                aForFrame,
+          DrawTarget*              aDrawTarget,
           float                    aFontSizeInflation,
           nsStretchDirection       aStretchDirection,
           const nsBoundingMetrics& aContainerSize,
@@ -123,8 +126,7 @@ public:
           bool                     aRTL);
 
   void
-  SetData(nsPresContext* aPresContext,
-          nsString&       aData);
+  SetData(nsString& aData);
 
   void
   GetData(nsString& aData) {
@@ -163,19 +165,11 @@ public:
   //
   // @param aStretchHint can be the value that will be passed to Stretch().
   // It is used to determine whether the operator is stretchy or a largeop.
-  // @param aMaxSize is the value of the "maxsize" attribute.
-  // @param aMaxSizeIsAbsolute indicates whether the aMaxSize is an absolute
-  // value in app units (true) or a multiplier of the base size (false).
   nscoord
-  GetMaxWidth(nsPresContext* aPresContext,
-              nsRenderingContext& aRenderingContext,
+  GetMaxWidth(nsIFrame* aForFrame,
+              DrawTarget* aDrawTarget,
               float aFontSizeInflation,
-              uint32_t aStretchHint = NS_STRETCH_NORMAL,
-              float aMaxSize = NS_MATHML_OPERATOR_SIZE_INFINITY,
-              // Perhaps just nsOperatorFlags aFlags.
-              // But need DisplayStyle for largeOp,
-              // or remove the largeop bit from flags.
-              bool aMaxSizeIsAbsolute = false);
+              uint32_t aStretchHint = NS_STRETCH_NORMAL);
 
   // Metrics that _exactly_ enclose the char. The char *must* have *already*
   // being stretched before you can call the GetBoundingMetrics() method.
@@ -210,10 +204,10 @@ private:
   nsRect             mRect;
   nsStretchDirection mDirection;
   nsBoundingMetrics  mBoundingMetrics;
-  nsStyleContext*    mStyleContext;
+  RefPtr<nsStyleContext> mStyleContext;
   // mGlyphs/mBmData are arrays describing the glyphs used to draw the operator.
   // See the drawing methods below.
-  nsAutoPtr<gfxTextRun> mGlyphs[4];
+  RefPtr<gfxTextRun> mGlyphs[4];
   nsBoundingMetrics     mBmData[4];
   // mUnscaledAscent is the actual ascent of the char.
   nscoord            mUnscaledAscent;
@@ -230,7 +224,7 @@ private:
   };
   DrawingMethod mDraw;
 
-  // mMirrored indicates whether the character is mirrored. 
+  // mMirrored indicates whether the character is mirrored.
   bool               mMirrored;
 
   class StretchEnumContext;
@@ -243,11 +237,11 @@ private:
                 const nsGlyphCode&      aGlyphCode,
                 const mozilla::FontFamilyList& aDefaultFamily,
                 nsFont&                 aFont,
-                nsRefPtr<gfxFontGroup>* aFontGroup);
+                RefPtr<gfxFontGroup>* aFontGroup);
 
   nsresult
-  StretchInternal(nsPresContext*           aPresContext,
-                  gfxContext*              aThebesContext,
+  StretchInternal(nsIFrame*                aForFrame,
+                  DrawTarget*              aDrawTarget,
                   float                    aFontSizeInflation,
                   nsStretchDirection&      aStretchDirection,
                   const nsBoundingMetrics& aContainerSize,
