@@ -1,4 +1,5 @@
-/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
+/* vim: set ts=8 sts=2 et sw=2 tw=80: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -9,7 +10,7 @@
 #include "mozilla/Attributes.h"
 #include "nsMathMLContainerFrame.h"
 #include "nsBlockFrame.h"
-#include "nsTableOuterFrame.h"
+#include "nsTableWrapperFrame.h"
 #include "nsTableRowFrame.h"
 #include "nsTableCellFrame.h"
 
@@ -17,57 +18,61 @@
 // <mtable> -- table or matrix
 //
 
-class nsMathMLmtableOuterFrame : public nsTableOuterFrame,
-                                 public nsMathMLFrame
+class nsMathMLmtableWrapperFrame final
+  : public nsTableWrapperFrame
+  , public nsMathMLFrame
 {
 public:
-  friend nsContainerFrame* NS_NewMathMLmtableOuterFrame(nsIPresShell* aPresShell,
-                                                        nsStyleContext* aContext);
+  friend nsContainerFrame*
+  NS_NewMathMLmtableOuterFrame(nsIPresShell*   aPresShell,
+                               nsStyleContext* aContext);
 
   NS_DECL_QUERYFRAME
-  NS_DECL_FRAMEARENA_HELPERS
+  NS_DECL_FRAMEARENA_HELPERS(nsMathMLmtableWrapperFrame)
 
-  // overloaded nsTableOuterFrame methods
+  // overloaded nsTableWrapperFrame methods
 
   virtual void
-  Reflow(nsPresContext*          aPresContext,
-         nsHTMLReflowMetrics&     aDesiredSize,
-         const nsHTMLReflowState& aReflowState,
+  Reflow(nsPresContext*           aPresContext,
+         ReflowOutput&     aDesiredSize,
+         const ReflowInput& aReflowInput,
          nsReflowStatus&          aStatus) override;
 
   virtual nsresult
   AttributeChanged(int32_t  aNameSpaceID,
-                   nsIAtom* aAttribute,
+                   nsAtom* aAttribute,
                    int32_t  aModType) override;
 
   virtual bool IsFrameOfType(uint32_t aFlags) const override
   {
-    return nsTableOuterFrame::IsFrameOfType(aFlags & ~(nsIFrame::eMathML));
+    return nsTableWrapperFrame::IsFrameOfType(aFlags & ~(nsIFrame::eMathML));
   }
 
 protected:
-  explicit nsMathMLmtableOuterFrame(nsStyleContext* aContext) : nsTableOuterFrame(aContext) {}
-  virtual ~nsMathMLmtableOuterFrame();
+  explicit nsMathMLmtableWrapperFrame(nsStyleContext* aContext)
+    : nsTableWrapperFrame(aContext, kClassID)
+  {}
+
+  virtual ~nsMathMLmtableWrapperFrame();
 
   // helper to find the row frame at a given index, positive or negative, e.g.,
   // 1..n means the first row down to the last row, -1..-n means the last row
   // up to the first row. Used for alignments that are relative to a given row
   nsIFrame*
-  GetRowFrameAt(nsPresContext* aPresContext,
-                int32_t         aRowIndex);
-}; // class nsMathMLmtableOuterFrame
+  GetRowFrameAt(int32_t aRowIndex);
+}; // class nsMathMLmtableWrapperFrame
 
 // --------------
 
-class nsMathMLmtableFrame : public nsTableFrame
+class nsMathMLmtableFrame final : public nsTableFrame
 {
 public:
-  NS_DECL_QUERYFRAME_TARGET(nsMathMLmtableFrame)
   NS_DECL_QUERYFRAME
-  NS_DECL_FRAMEARENA_HELPERS
+  NS_DECL_FRAMEARENA_HELPERS(nsMathMLmtableFrame)
 
-  friend nsContainerFrame* NS_NewMathMLmtableFrame(nsIPresShell* aPresShell,
-                                                   nsStyleContext* aContext);
+  friend nsContainerFrame*
+  NS_NewMathMLmtableFrame(nsIPresShell*   aPresShell,
+                          nsStyleContext* aContext);
 
   // Overloaded nsTableFrame methods
 
@@ -93,8 +98,7 @@ public:
   }
 
   virtual void
-  RemoveFrame(ChildListID aListID,
-              nsIFrame* aOldFrame) override
+  RemoveFrame(ChildListID aListID, nsIFrame* aOldFrame) override
   {
     nsTableFrame::RemoveFrame(aListID, aOldFrame);
     RestyleTable();
@@ -110,23 +114,23 @@ public:
   // safer (albeit grossly suboptimal) to just relayout the whole thing.
   void RestyleTable();
 
-  /** helper to get the cell spacing X style value */
-  nscoord GetCellSpacingX(int32_t aColIndex) override;
+  /** helper to get the column spacing style value */
+  nscoord GetColSpacing(int32_t aColIndex) override;
 
   /** Sums the combined cell spacing between the columns aStartColIndex to
    *  aEndColIndex.
    */
-  nscoord GetCellSpacingX(int32_t aStartColIndex,
-                          int32_t aEndColIndex) override;
+  nscoord GetColSpacing(int32_t aStartColIndex,
+                        int32_t aEndColIndex) override;
 
-  /** helper to get the cell spacing Y style value */
-  nscoord GetCellSpacingY(int32_t aRowIndex) override;
+  /** helper to get the row spacing style value */
+  nscoord GetRowSpacing(int32_t aRowIndex) override;
 
   /** Sums the combined cell spacing between the rows aStartRowIndex to
    *  aEndRowIndex.
    */
-  nscoord GetCellSpacingY(int32_t aStartRowIndex,
-                          int32_t aEndRowIndex) override;
+  nscoord GetRowSpacing(int32_t aStartRowIndex,
+                        int32_t aEndRowIndex) override;
 
   void SetColSpacingArray(const nsTArray<nscoord>& aColSpacing)
   {
@@ -150,14 +154,16 @@ public:
    * approach is used if the user specifies at least one of those attributes.
    */
   void SetUseCSSSpacing();
-
-  bool GetUseCSSSpacing()
-  {
-    return mUseCSSSpacing;
-  }
+  bool GetUseCSSSpacing() { return mUseCSSSpacing; }
 
 protected:
-  explicit nsMathMLmtableFrame(nsStyleContext* aContext) : nsTableFrame(aContext) {}
+  explicit nsMathMLmtableFrame(nsStyleContext* aContext)
+    : nsTableFrame(aContext, kClassID)
+    , mFrameSpacingX(0)
+    , mFrameSpacingY(0)
+    , mUseCSSSpacing(false)
+  {}
+
   virtual ~nsMathMLmtableFrame();
 
 private:
@@ -170,19 +176,20 @@ private:
 
 // --------------
 
-class nsMathMLmtrFrame : public nsTableRowFrame
+class nsMathMLmtrFrame final : public nsTableRowFrame
 {
 public:
-  NS_DECL_FRAMEARENA_HELPERS
+  NS_DECL_FRAMEARENA_HELPERS(nsMathMLmtrFrame)
 
-  friend nsContainerFrame* NS_NewMathMLmtrFrame(nsIPresShell* aPresShell,
-                                                nsStyleContext* aContext);
+  friend nsContainerFrame*
+  NS_NewMathMLmtrFrame(nsIPresShell*   aPresShell,
+                       nsStyleContext* aContext);
 
   // overloaded nsTableRowFrame methods
 
   virtual nsresult
   AttributeChanged(int32_t  aNameSpaceID,
-                   nsIAtom* aAttribute,
+                   nsAtom* aAttribute,
                    int32_t  aModType) override;
 
   virtual void
@@ -194,8 +201,8 @@ public:
   }
 
   virtual void
-  InsertFrames(ChildListID aListID,
-               nsIFrame* aPrevFrame,
+  InsertFrames(ChildListID  aListID,
+               nsIFrame*    aPrevFrame,
                nsFrameList& aFrameList) override
   {
     nsTableRowFrame::InsertFrames(aListID, aPrevFrame, aFrameList);
@@ -203,8 +210,7 @@ public:
   }
 
   virtual void
-  RemoveFrame(ChildListID aListID,
-              nsIFrame* aOldFrame) override
+  RemoveFrame(ChildListID aListID, nsIFrame* aOldFrame) override
   {
     nsTableRowFrame::RemoveFrame(aListID, aOldFrame);
     RestyleTable();
@@ -218,7 +224,7 @@ public:
   // helper to restyle and reflow the table -- @see nsMathMLmtableFrame.
   void RestyleTable()
   {
-    nsTableFrame* tableFrame = nsTableFrame::GetTableFrame(this);
+    nsTableFrame* tableFrame = GetTableFrame();
     if (tableFrame && tableFrame->IsFrameOfType(nsIFrame::eMathML)) {
       // relayout the table
       ((nsMathMLmtableFrame*)tableFrame)->RestyleTable();
@@ -226,7 +232,10 @@ public:
   }
 
 protected:
-  explicit nsMathMLmtrFrame(nsStyleContext* aContext) : nsTableRowFrame(aContext) {}
+  explicit nsMathMLmtrFrame(nsStyleContext* aContext)
+    : nsTableRowFrame(aContext, kClassID)
+  {}
+
   virtual ~nsMathMLmtrFrame();
 }; // class nsMathMLmtrFrame
 
@@ -235,10 +244,12 @@ protected:
 class nsMathMLmtdFrame : public nsTableCellFrame
 {
 public:
-  NS_DECL_FRAMEARENA_HELPERS
+  NS_DECL_FRAMEARENA_HELPERS(nsMathMLmtdFrame)
 
-  friend nsContainerFrame* NS_NewMathMLmtdFrame(nsIPresShell* aPresShell,
-                                                nsStyleContext* aContext);
+  friend nsContainerFrame*
+  NS_NewMathMLmtdFrame(nsIPresShell*   aPresShell,
+                       nsStyleContext* aContext,
+                       nsTableFrame*   aTableFrame);
 
   // overloaded nsTableCellFrame methods
 
@@ -248,47 +259,53 @@ public:
 
   virtual nsresult
   AttributeChanged(int32_t  aNameSpaceID,
-                   nsIAtom* aAttribute,
+                   nsAtom* aAttribute,
                    int32_t  aModType) override;
 
   virtual uint8_t GetVerticalAlign() const override;
-  virtual nsresult ProcessBorders(nsTableFrame* aFrame,
-                                  nsDisplayListBuilder* aBuilder,
+  virtual nsresult ProcessBorders(nsTableFrame*           aFrame,
+                                  nsDisplayListBuilder*   aBuilder,
                                   const nsDisplayListSet& aLists) override;
 
-  virtual int32_t GetRowSpan() override;
-  virtual int32_t GetColSpan() override;
   virtual bool IsFrameOfType(uint32_t aFlags) const override
   {
     return nsTableCellFrame::IsFrameOfType(aFlags & ~(nsIFrame::eMathML));
   }
 
-  virtual nsMargin* GetBorderWidth(nsMargin& aBorder) const override;
+  virtual LogicalMargin GetBorderWidth(WritingMode aWM) const override;
 
   virtual nsMargin GetBorderOverflow() override;
 
 protected:
-  explicit nsMathMLmtdFrame(nsStyleContext* aContext) : nsTableCellFrame(aContext) {}
+  nsMathMLmtdFrame(nsStyleContext* aContext, nsTableFrame* aTableFrame)
+    : nsTableCellFrame(aContext, aTableFrame, kClassID)
+  {
+  }
+
   virtual ~nsMathMLmtdFrame();
 }; // class nsMathMLmtdFrame
 
 // --------------
 
-class nsMathMLmtdInnerFrame : public nsBlockFrame,
-                              public nsMathMLFrame {
+class nsMathMLmtdInnerFrame final
+  : public nsBlockFrame
+  , public nsMathMLFrame
+{
 public:
-  friend nsContainerFrame* NS_NewMathMLmtdInnerFrame(nsIPresShell* aPresShell, nsStyleContext* aContext);
+  friend nsContainerFrame*
+  NS_NewMathMLmtdInnerFrame(nsIPresShell*   aPresShell,
+                            nsStyleContext* aContext);
 
   NS_DECL_QUERYFRAME
-  NS_DECL_FRAMEARENA_HELPERS
+  NS_DECL_FRAMEARENA_HELPERS(nsMathMLmtdInnerFrame)
 
   // Overloaded nsIMathMLFrame methods
 
   NS_IMETHOD
-  UpdatePresentationDataFromChildAt(int32_t         aFirstIndex,
-                                    int32_t         aLastIndex,
-                                    uint32_t        aFlagsValues,
-                                    uint32_t        aFlagsToUpdate) override
+  UpdatePresentationDataFromChildAt(int32_t  aFirstIndex,
+                                    int32_t  aLastIndex,
+                                    uint32_t aFlagsValues,
+                                    uint32_t aFlagsToUpdate) override
   {
     nsMathMLContainerFrame::PropagatePresentationDataFromChildAt(this,
       aFirstIndex, aLastIndex, aFlagsValues, aFlagsToUpdate);
@@ -296,9 +313,9 @@ public:
   }
 
   virtual void
-  Reflow(nsPresContext*          aPresContext,
-         nsHTMLReflowMetrics&     aDesiredSize,
-         const nsHTMLReflowState& aReflowState,
+  Reflow(nsPresContext*           aPresContext,
+         ReflowOutput&     aDesiredSize,
+         const ReflowInput& aReflowInput,
          nsReflowStatus&          aStatus) override;
 
   virtual bool IsFrameOfType(uint32_t aFlags) const override
@@ -311,8 +328,10 @@ public:
   virtual void DidSetStyleContext(nsStyleContext* aOldStyleContext) override;
 
   bool
-  IsMrowLike() override {
-    return mFrames.FirstChild() != mFrames.LastChild() ||
+  IsMrowLike() override
+  {
+    return mFrames.FirstChild() !=
+           mFrames.LastChild() ||
            !mFrames.FirstChild();
   }
 
