@@ -8,46 +8,25 @@
 #include "mozilla/Attributes.h"
 #include "nscore.h"
 #include "nsContainerFrame.h"
-#include "nsIAtom.h"
+#include "nsAtom.h"
 #include "nsILineIterator.h"
-#include "nsTablePainter.h"
 #include "nsTArray.h"
+#include "nsTableFrame.h"
+#include "mozilla/WritingModes.h"
 
-class nsTableFrame;
 class nsTableRowFrame;
-class nsTableCellFrame;
-
-struct nsRowGroupReflowState {
-  const nsHTMLReflowState& reflowState;  // Our reflow state
-
-  nsTableFrame* tableFrame;
-
-  // The available size (computed from the parent)
-  nsSize availSize;
-
-  // Running y-offset
-  nscoord y;
-
-  nsRowGroupReflowState(const nsHTMLReflowState& aReflowState,
-                        nsTableFrame*            aTableFrame)
-      :reflowState(aReflowState), tableFrame(aTableFrame)
-  {
-    availSize.width  = reflowState.AvailableWidth();
-    availSize.height = reflowState.AvailableHeight();
-    y = 0;  
-  }
-
-  ~nsRowGroupReflowState() {}
-};
+namespace mozilla {
+struct TableRowGroupReflowInput;
+} // namespace mozilla
 
 #define MIN_ROWS_NEEDING_CURSOR 20
 
 /**
- * nsTableRowGroupFrame is the frame that maps row groups 
+ * nsTableRowGroupFrame is the frame that maps row groups
  * (HTML tags THEAD, TFOOT, and TBODY). This class cannot be reused
- * outside of an nsTableFrame.  It assumes that its parent is an nsTableFrame, and 
+ * outside of an nsTableFrame.  It assumes that its parent is an nsTableFrame, and
  * its children are nsTableRowFrames.
- * 
+ *
  * @see nsTableFrame
  * @see nsTableRowFrame
  */
@@ -55,10 +34,11 @@ class nsTableRowGroupFrame final
   : public nsContainerFrame
   , public nsILineIterator
 {
+  using TableRowGroupReflowInput = mozilla::TableRowGroupReflowInput;
+
 public:
-  NS_DECL_QUERYFRAME_TARGET(nsTableRowGroupFrame)
   NS_DECL_QUERYFRAME
-  NS_DECL_FRAMEARENA_HELPERS
+  NS_DECL_FRAMEARENA_HELPERS(nsTableRowGroupFrame)
 
   /** instantiate a new instance of nsTableRowFrame.
     * @param aPresShell the pres shell for this frame
@@ -69,11 +49,22 @@ public:
                                                         nsStyleContext* aContext);
   virtual ~nsTableRowGroupFrame();
 
-  virtual void DestroyFrom(nsIFrame* aDestructRoot) override;
+  // nsIFrame overrides
+  virtual void Init(nsIContent*       aContent,
+                    nsContainerFrame* aParent,
+                    nsIFrame*         aPrevInFlow) override
+  {
+    nsContainerFrame::Init(aContent, aParent, aPrevInFlow);
+    if (!aPrevInFlow) {
+      mWritingMode = GetTableFrame()->GetWritingMode();
+    }
+  }
+
+  virtual void DestroyFrom(nsIFrame* aDestructRoot, PostDestroyData& aPostDestroyData) override;
 
   /** @see nsIFrame::DidSetStyleContext */
   virtual void DidSetStyleContext(nsStyleContext* aOldStyleContext) override;
-  
+
   virtual void AppendFrames(ChildListID     aListID,
                             nsFrameList&    aFrameList) override;
   virtual void InsertFrames(ChildListID     aListID,
@@ -87,38 +78,38 @@ public:
   virtual nsMargin GetUsedPadding() const override;
 
   virtual void BuildDisplayList(nsDisplayListBuilder*   aBuilder,
-                                const nsRect&           aDirtyRect,
                                 const nsDisplayListSet& aLists) override;
 
    /** calls Reflow for all of its child rows.
-    * Rows are all set to the same width and stacked vertically.
+    * Rows are all set to the same isize and stacked in the block direction.
     * <P> rows are not split unless absolutely necessary.
     *
-    * @param aDesiredSize width set to width of rows, height set to 
-    *                     sum of height of rows that fit in aMaxSize.height.
+    * @param aDesiredSize isize set to isize of rows, bsize set to
+    *                     sum of bsize of rows that fit in AvailableBSize.
     *
     * @see nsIFrame::Reflow
     */
   virtual void Reflow(nsPresContext*           aPresContext,
-                      nsHTMLReflowMetrics&     aDesiredSize,
-                      const nsHTMLReflowState& aReflowState,
+                      ReflowOutput&     aDesiredSize,
+                      const ReflowInput& aReflowInput,
                       nsReflowStatus&          aStatus) override;
 
-  virtual bool UpdateOverflow() override;
-
-  /**
-   * Get the "type" of the frame
-   *
-   * @see nsGkAtoms::tableRowGroupFrame
-   */
-  virtual nsIAtom* GetType() const override;
-
-  nsTableRowFrame* GetFirstRow();
+  virtual bool ComputeCustomOverflow(nsOverflowAreas& aOverflowAreas) override;
 
 #ifdef DEBUG_FRAME_DUMP
   virtual nsresult GetFrameName(nsAString& aResult) const override;
 #endif
 
+  nsTableRowFrame* GetFirstRow();
+  nsTableRowFrame* GetLastRow();
+
+  nsTableFrame* GetTableFrame() const
+  {
+    nsIFrame* parent = GetParent();
+    MOZ_ASSERT(parent && parent->IsTableFrame());
+    return static_cast<nsTableFrame*>(parent);
+  }
+
   /** return the number of child rows (not necessarily == number of child frames) */
   int32_t GetRowCount();
 
@@ -127,13 +118,26 @@ public:
     */
   int32_t GetStartRowIndex();
 
-  /** Adjust the row indices of all rows  whose index is >= aRowIndex.  
+  /** Adjust the row indices of all rows  whose index is >= aRowIndex.
     * @param aRowIndex   - start adjusting with this index
     * @param aAdjustment - shift the row index by this amount
     */
   void AdjustRowIndices(int32_t   aRowIndex,
                         int32_t   anAdjustment);
 
+  // See nsTableFrame.h
+  int32_t GetAdjustmentForStoredIndex(int32_t aStoredIndex);
+
+  /* mark rows starting from aStartRowFrame to the next 'aNumRowsToRemove-1'
+   * number of rows as deleted
+   */
+  void MarkRowsAsDeleted(nsTableRowFrame& aStartRowFrame,
+                         int32_t          aNumRowsToDelete);
+
+  // See nsTableFrame.h
+  void AddDeletedRowIndex(int32_t aDeletedRowStoredIndex);
+
+
   /**
    * Used for header and footer row group frames that are repeated when
    * splitting a table frame.
@@ -143,53 +147,56 @@ public:
    * @param aHeaderFooterFrame the original header or footer row group frame
    * that was repeated
    */
-  nsresult  InitRepeatedFrame(nsPresContext*        aPresContext,
-                              nsTableRowGroupFrame* aHeaderFooterFrame);
+  nsresult  InitRepeatedFrame(nsTableRowGroupFrame* aHeaderFooterFrame);
+
 
-  
   /**
-   * Get the total height of all the row rects
+   * Get the total bsize of all the row rects
    */
-  nscoord GetHeightBasis(const nsHTMLReflowState& aReflowState);
-  
-  nsMargin* GetBCBorderWidth(nsMargin& aBorder);
+  nscoord GetBSizeBasis(const ReflowInput& aReflowInput);
+
+  mozilla::LogicalMargin GetBCBorderWidth(mozilla::WritingMode aWM);
 
   /**
    * Gets inner border widths before collapsing with cell borders
-   * Caller must get top border from previous row group or from table
-   * GetContinuousBCBorderWidth will not overwrite aBorder.top
+   * Caller must get bstart border from previous row group or from table
+   * GetContinuousBCBorderWidth will not overwrite aBorder.BStart()
    * see nsTablePainter about continuous borders
    */
-  void GetContinuousBCBorderWidth(nsMargin& aBorder);
+  void GetContinuousBCBorderWidth(mozilla::WritingMode aWM,
+                                  mozilla::LogicalMargin& aBorder);
+
   /**
    * Sets full border widths before collapsing with cell borders
-   * @param aForSide - side to set; only right, left, and bottom valid
+   * @param aForSide - side to set; only IEnd, IStart, BEnd are valid
    */
-  void SetContinuousBCBorderWidth(uint8_t     aForSide,
+  void SetContinuousBCBorderWidth(mozilla::LogicalSide aForSide,
                                   BCPixelSize aPixelValue);
   /**
-    * Adjust to the effect of visibibility:collapse on the row group and
+    * Adjust to the effect of visibility:collapse on the row group and
     * its children
-    * @return              additional shift upward that should be applied to
-    *                      subsequent rowgroups due to rows and this rowgroup
-    *                      being collapsed
-    * @param aYTotalOffset the total amount that the rowgroup is shifted up
-    * @param aWidth        new width of the rowgroup
+    * @return              additional shift bstart-wards that should be applied
+    *                      to subsequent rowgroups due to rows and this
+    *                      rowgroup being collapsed
+    * @param aBTotalOffset the total amount that the rowgroup is shifted
+    * @param aISize        new isize of the rowgroup
+    * @param aWM           the table's writing mode
     */
-  nscoord CollapseRowGroupIfNecessary(nscoord aYTotalOffset,
-                                      nscoord aWidth);
+  nscoord CollapseRowGroupIfNecessary(nscoord aBTotalOffset,
+                                      nscoord aISize,
+                                      mozilla::WritingMode aWM);
 
 // nsILineIterator methods
 public:
   virtual void DisposeLineIterator() override { }
 
-  // The table row is the equivalent to a line in block layout. 
+  // The table row is the equivalent to a line in block layout.
   // The nsILineIterator assumes that a line resides in a block, this role is
   // fullfilled by the row group. Rows in table are counted relative to the
   // table. The row index of row corresponds to the cellmap coordinates. The
   // line index with respect to a row group can be computed by substracting the
   // row index of the first row in the row group.
-   
+
   /** Get the number of rows in a row group
     * @return the number of lines in a row group
     */
@@ -199,11 +206,11 @@ public:
     * @return true if the table is rtl
     */
   virtual bool GetDirection() override;
-  
-  /** Return structural information about a line. 
+
+  /** Return structural information about a line.
     * @param aLineNumber       - the index of the row relative to the row group
     *                            If the line-number is invalid then
-    *                            aFirstFrameOnLine will be nullptr and 
+    *                            aFirstFrameOnLine will be nullptr and
     *                            aNumFramesOnLine will be zero.
     * @param aFirstFrameOnLine - the first cell frame that originates in row
     *                            with a rowindex that matches a line number
@@ -215,7 +222,7 @@ public:
                      nsIFrame** aFirstFrameOnLine,
                      int32_t* aNumFramesOnLine,
                      nsRect& aLineBounds) override;
-  
+
   /** Given a frame that's a child of the rowgroup, find which line its on.
     * @param aFrame       - frame, should be a row
     * @param aStartLine   - minimal index to return
@@ -255,11 +262,11 @@ public:
                             nsIFrame                 **aFirstVisual,
                             nsIFrame                 **aLastVisual) override;
 
-  /** Find the next originating cell frame that originates in the row.    
+  /** Find the next originating cell frame that originates in the row.
     * @param aFrame      - cell frame to start with, will return the next cell
     *                      originating in a row
     * @param aLineNumber - the index of the row relative to the table
-    */  
+    */
   NS_IMETHOD GetNextSiblingOnLine(nsIFrame*& aFrame, int32_t aLineNumber) override;
 
   // row cursor methods to speed up searching for the row(s)
@@ -277,13 +284,13 @@ public:
     uint32_t            mCursorIndex;
     nscoord             mOverflowAbove;
     nscoord             mOverflowBelow;
-    
+
     FrameCursorData()
       : mFrames(MIN_ROWS_NEEDING_CURSOR), mCursorIndex(0), mOverflowAbove(0),
         mOverflowBelow(0) {}
 
     bool AppendFrame(nsIFrame* aFrame);
-    
+
     void FinishBuildingCursor() {
       mFrames.Compact();
     }
@@ -328,53 +335,55 @@ public:
 protected:
   explicit nsTableRowGroupFrame(nsStyleContext* aContext);
 
-  void InitChildReflowState(nsPresContext&     aPresContext, 
+  void InitChildReflowInput(nsPresContext&     aPresContext,
                             bool               aBorderCollapse,
-                            nsHTMLReflowState& aReflowState);
-  
-  virtual LogicalSides GetLogicalSkipSides(const nsHTMLReflowState* aReflowState = nullptr) const override;
+                            ReflowInput& aReflowInput);
+
+  virtual LogicalSides GetLogicalSkipSides(const ReflowInput* aReflowInput = nullptr) const override;
 
   void PlaceChild(nsPresContext*         aPresContext,
-                  nsRowGroupReflowState& aReflowState,
+                  TableRowGroupReflowInput& aReflowInput,
                   nsIFrame*              aKidFrame,
-                  nsPoint                aKidPosition,
-                  nsHTMLReflowMetrics&   aDesiredSize,
+                  mozilla::WritingMode   aWM,
+                  const mozilla::LogicalPoint& aKidPosition,
+                  const nsSize&          aContainerSize,
+                  ReflowOutput&   aDesiredSize,
                   const nsRect&          aOriginalKidRect,
                   const nsRect&          aOriginalKidVisualOverflow);
 
-  void CalculateRowHeights(nsPresContext*           aPresContext, 
-                           nsHTMLReflowMetrics&     aDesiredSize,
-                           const nsHTMLReflowState& aReflowState);
+  void CalculateRowBSizes(nsPresContext*           aPresContext,
+                          ReflowOutput&     aDesiredSize,
+                          const ReflowInput& aReflowInput);
 
-  void DidResizeRows(nsHTMLReflowMetrics& aDesiredSize);
+  void DidResizeRows(ReflowOutput& aDesiredSize);
 
-  void SlideChild(nsRowGroupReflowState& aReflowState,
+  void SlideChild(TableRowGroupReflowInput& aReflowInput,
                   nsIFrame*              aKidFrame);
-  
+
   /**
    * Reflow the frames we've already created
    *
    * @param   aPresContext presentation context to use
-   * @param   aReflowState current inline state
+   * @param   aReflowInput current inline state
    */
   void ReflowChildren(nsPresContext*         aPresContext,
-                      nsHTMLReflowMetrics&   aDesiredSize,
-                      nsRowGroupReflowState& aReflowState,
+                      ReflowOutput&   aDesiredSize,
+                      TableRowGroupReflowInput& aReflowInput,
                       nsReflowStatus&        aStatus,
                       bool*                aPageBreakBeforeEnd = nullptr);
 
   nsresult SplitRowGroup(nsPresContext*           aPresContext,
-                         nsHTMLReflowMetrics&     aDesiredSize,
-                         const nsHTMLReflowState& aReflowState,
+                         ReflowOutput&     aDesiredSize,
+                         const ReflowInput& aReflowInput,
                          nsTableFrame*            aTableFrame,
                          nsReflowStatus&          aStatus,
                          bool                     aRowForcedPageBreak);
 
   void SplitSpanningCells(nsPresContext&           aPresContext,
-                          const nsHTMLReflowState& aReflowState,
+                          const ReflowInput& aReflowInput,
                           nsTableFrame&            aTableFrame,
-                          nsTableRowFrame&         aFirstRow, 
-                          nsTableRowFrame&         aLastRow,  
+                          nsTableRowFrame&         aFirstRow,
+                          nsTableRowFrame&         aLastRow,
                           bool                     aFirstRowIsTopOfPage,
                           nscoord                  aSpanningRowBottom,
                           nsTableRowFrame*&        aContRowFrame,
@@ -385,25 +394,25 @@ protected:
                                 nsIFrame&      aRowFrame,
                                 nsIFrame**     aContRowFrame);
 
-  bool IsSimpleRowFrame(nsTableFrame* aTableFrame, 
-                          nsIFrame*     aFrame);
+  bool IsSimpleRowFrame(nsTableFrame* aTableFrame,
+                        nsTableRowFrame* aRowFrame);
 
   void GetNextRowSibling(nsIFrame** aRowFrame);
 
   void UndoContinuedRow(nsPresContext*   aPresContext,
                         nsTableRowFrame* aRow);
-                        
+
 private:
   // border widths in pixels in the collapsing border model
-  BCPixelSize mRightContBorderWidth;
-  BCPixelSize mBottomContBorderWidth;
-  BCPixelSize mLeftContBorderWidth;
+  BCPixelSize mIEndContBorderWidth;
+  BCPixelSize mBEndContBorderWidth;
+  BCPixelSize mIStartContBorderWidth;
 
 public:
   bool IsRepeatable() const;
-  void   SetRepeatable(bool aRepeatable);
-  bool HasStyleHeight() const;
-  void   SetHasStyleHeight(bool aValue);
+  void SetRepeatable(bool aRepeatable);
+  bool HasStyleBSize() const;
+  void SetHasStyleBSize(bool aValue);
   bool HasInternalBreakBefore() const;
   bool HasInternalBreakAfter() const;
 };
@@ -411,42 +420,42 @@ public:
 
 inline bool nsTableRowGroupFrame::IsRepeatable() const
 {
-  return (mState & NS_ROWGROUP_REPEATABLE) == NS_ROWGROUP_REPEATABLE;
+  return HasAnyStateBits(NS_ROWGROUP_REPEATABLE);
 }
 
 inline void nsTableRowGroupFrame::SetRepeatable(bool aRepeatable)
 {
   if (aRepeatable) {
-    mState |= NS_ROWGROUP_REPEATABLE;
+    AddStateBits(NS_ROWGROUP_REPEATABLE);
   } else {
-    mState &= ~NS_ROWGROUP_REPEATABLE;
+    RemoveStateBits(NS_ROWGROUP_REPEATABLE);
   }
 }
 
-inline bool nsTableRowGroupFrame::HasStyleHeight() const
+inline bool nsTableRowGroupFrame::HasStyleBSize() const
 {
-  return (mState & NS_ROWGROUP_HAS_STYLE_HEIGHT) == NS_ROWGROUP_HAS_STYLE_HEIGHT;
+  return HasAnyStateBits(NS_ROWGROUP_HAS_STYLE_BSIZE);
 }
 
-inline void nsTableRowGroupFrame::SetHasStyleHeight(bool aValue)
+inline void nsTableRowGroupFrame::SetHasStyleBSize(bool aValue)
 {
   if (aValue) {
-    mState |= NS_ROWGROUP_HAS_STYLE_HEIGHT;
+    AddStateBits(NS_ROWGROUP_HAS_STYLE_BSIZE);
   } else {
-    mState &= ~NS_ROWGROUP_HAS_STYLE_HEIGHT;
+    RemoveStateBits(NS_ROWGROUP_HAS_STYLE_BSIZE);
   }
 }
 
 inline void
-nsTableRowGroupFrame::GetContinuousBCBorderWidth(nsMargin& aBorder)
+nsTableRowGroupFrame::GetContinuousBCBorderWidth(mozilla::WritingMode aWM,
+                                                 mozilla::LogicalMargin& aBorder)
 {
-  int32_t aPixelsToTwips = nsPresContext::AppUnitsPerCSSPixel();
-  aBorder.right = BC_BORDER_LEFT_HALF_COORD(aPixelsToTwips,
-                                            mRightContBorderWidth);
-  aBorder.bottom = BC_BORDER_TOP_HALF_COORD(aPixelsToTwips,
-                                            mBottomContBorderWidth);
-  aBorder.left = BC_BORDER_RIGHT_HALF_COORD(aPixelsToTwips,
-                                            mLeftContBorderWidth);
-  return;
+  int32_t d2a = PresContext()->AppUnitsPerDevPixel();
+  aBorder.IEnd(aWM) = BC_BORDER_START_HALF_COORD(d2a,
+                                                 mIEndContBorderWidth);
+  aBorder.BEnd(aWM) = BC_BORDER_START_HALF_COORD(d2a,
+                                                 mBEndContBorderWidth);
+  aBorder.IStart(aWM) = BC_BORDER_END_HALF_COORD(d2a,
+                                                 mIStartContBorderWidth);
 }
 #endif
