@@ -24,17 +24,15 @@
  * multiple HAWK services should instantiate one HawkClient per service.
  */
 
-this.EXPORTED_SYMBOLS = ["HawkClient"];
+var EXPORTED_SYMBOLS = ["HawkClient"];
 
-var {interfaces: Ci, utils: Cu} = Components;
-
-Cu.import("resource://services-crypto/utils.js");
-Cu.import("resource://services-common/hawkrequest.js");
-Cu.import("resource://services-common/observers.js");
-Cu.import("resource://gre/modules/Promise.jsm");
-Cu.import("resource://gre/modules/Log.jsm");
-Cu.import("resource://gre/modules/XPCOMUtils.jsm");
-Cu.import("resource://gre/modules/Services.jsm");
+ChromeUtils.import("resource://services-crypto/utils.js");
+ChromeUtils.import("resource://services-common/hawkrequest.js");
+ChromeUtils.import("resource://services-common/observers.js");
+ChromeUtils.import("resource://gre/modules/PromiseUtils.jsm");
+ChromeUtils.import("resource://gre/modules/Log.jsm");
+ChromeUtils.import("resource://gre/modules/XPCOMUtils.jsm");
+ChromeUtils.import("resource://gre/modules/Services.jsm");
 
 // log.appender.dump should be one of "Fatal", "Error", "Warn", "Info", "Config",
 // "Debug", "Trace" or "All". If none is specified, "Error" will be used by
@@ -71,7 +69,7 @@ XPCOMUtils.defineLazyGetter(this, "log", function() {
 
 // A boolean to indicate if personally identifiable information (or anything
 // else sensitive, such as credentials) should be logged.
-XPCOMUtils.defineLazyGetter(this, 'logPII', function() {
+XPCOMUtils.defineLazyGetter(this, "logPII", function() {
   try {
     return Services.prefs.getBoolPref(PREF_LOG_SENSITIVE_DETAILS);
   } catch (_) {
@@ -90,13 +88,13 @@ XPCOMUtils.defineLazyGetter(this, 'logPII', function() {
  * @param host
  *        The url of the host
  */
-this.HawkClient = function(host) {
+var HawkClient = function(host) {
   this.host = host;
 
   // Clock offset in milliseconds between our client's clock and the date
   // reported in responses from our host.
   this._localtimeOffsetMsec = 0;
-}
+};
 
 this.HawkClient.prototype = {
 
@@ -114,9 +112,9 @@ this.HawkClient.prototype = {
    * @param error
    *        A string or object describing the error
    */
-  _constructError: function(restResponse, error) {
+  _constructError(restResponse, error) {
     let errorObj = {
-      error: error,
+      error,
       // This object is likely to be JSON.stringify'd, but neither Error()
       // objects nor Components.Exception objects do the right thing there,
       // so we add a new element which is simply the .toString() version of
@@ -156,12 +154,12 @@ this.HawkClient.prototype = {
    * For HAWK clock skew and replay protection, see
    * https://github.com/hueniverse/hawk#replay-protection
    */
-  _updateClockOffset: function(dateString) {
+  _updateClockOffset(dateString) {
     try {
       let serverDateMsec = Date.parse(dateString);
       this._localtimeOffsetMsec = serverDateMsec - this.now();
       log.debug("Clock offset vs " + this.host + ": " + this._localtimeOffsetMsec);
-    } catch(err) {
+    } catch (err) {
       log.warn("Bad date header in server response: " + dateString);
     }
   },
@@ -180,7 +178,7 @@ this.HawkClient.prototype = {
   /*
    * return current time in milliseconds
    */
-  now: function() {
+  now() {
     return Date.now();
   },
 
@@ -203,11 +201,11 @@ this.HawkClient.prototype = {
    *        as JSON and contains an 'error' property, the promise will be
    *        rejected with this JSON-parsed response.
    */
-  request: function(path, method, credentials=null, payloadObj={}, extraHeaders = {},
-                    retryOK=true) {
+  request(path, method, credentials = null, payloadObj = {}, extraHeaders = {},
+                    retryOK = true) {
     method = method.toLowerCase();
 
-    let deferred = Promise.defer();
+    let deferred = PromiseUtils.defer();
     let uri = this.host + path;
     let self = this;
 
@@ -240,17 +238,18 @@ this.HawkClient.prototype = {
       if (error) {
         // When things really blow up, reconstruct an error object that follows
         // the general format of the server on error responses.
-        return deferred.reject(self._constructError(restResponse, error));
+        deferred.reject(self._constructError(restResponse, error));
+        return;
       }
 
-      self._updateClockOffset(restResponse.headers["date"]);
+      self._updateClockOffset(restResponse.headers.date);
 
       if (status === 401 && retryOK && !("retry-after" in restResponse.headers)) {
         // Retry once if we were rejected due to a bad timestamp.
         // Clock offset is adjusted already in the top of this function.
         log.debug("Received 401 for " + path + ": retrying");
-        return deferred.resolve(
-            self.request(path, method, credentials, payloadObj, extraHeaders, false));
+        deferred.resolve(self.request(path, method, credentials, payloadObj, extraHeaders, false));
+        return;
       }
 
       // If the server returned a json error message, use it in the rejection
@@ -263,19 +262,21 @@ this.HawkClient.prototype = {
       let jsonResponse = {};
       try {
         jsonResponse = JSON.parse(restResponse.body);
-      } catch(notJSON) {}
+      } catch (notJSON) {}
 
       let okResponse = (200 <= status && status < 300);
       if (!okResponse || jsonResponse.error) {
         if (jsonResponse.error) {
-          return deferred.reject(jsonResponse);
+          deferred.reject(jsonResponse);
+        } else {
+          deferred.reject(self._constructError(restResponse, "Request failed"));
         }
-        return deferred.reject(self._constructError(restResponse, "Request failed"));
+        return;
       }
       // It's up to the caller to know how to decode the response.
       // We just return the whole response.
       deferred.resolve(this.response);
-    };
+    }
 
     function onComplete(error) {
       try {
@@ -319,7 +320,7 @@ this.HawkClient.prototype = {
   observerPrefix: null,
 
   // Given an optional header value, notify that a backoff has been requested.
-  _maybeNotifyBackoff: function (response, headerName) {
+  _maybeNotifyBackoff(response, headerName) {
     if (!this.observerPrefix || !response.headers) {
       return;
     }
@@ -339,8 +340,8 @@ this.HawkClient.prototype = {
   },
 
   // override points for testing.
-  newHAWKAuthenticatedRESTRequest: function(uri, credentials, extra) {
+  newHAWKAuthenticatedRESTRequest(uri, credentials, extra) {
     return new HAWKAuthenticatedRESTRequest(uri, credentials, extra);
   },
 
-}
+};
