@@ -1,8 +1,9 @@
 /* Any copyright is dedicated to the Public Domain.
    http://creativecommons.org/publicdomain/zero/1.0/ */
 
-Cu.import("resource://services-common/kinto-offline-client.js");
-Cu.import("resource://testing-common/httpd.js");
+ChromeUtils.import("resource://services-common/kinto-offline-client.js");
+ChromeUtils.import("resource://services-common/kinto-storage-adapter.js");
+ChromeUtils.import("resource://testing-common/httpd.js");
 
 const BinaryInputStream = Components.Constructor("@mozilla.org/binaryinputstream;1",
   "nsIBinaryInputStream", "setInputStream");
@@ -10,51 +11,51 @@ const BinaryInputStream = Components.Constructor("@mozilla.org/binaryinputstream
 var server;
 
 // set up what we need to make storage adapters
-const Kinto = loadKinto();
-const IceCatAdapter = Kinto.adapters.IceCatAdapter;
 const kintoFilename = "kinto.sqlite";
 
-let kintoClient;
-
-function do_get_kinto_collection() {
-  if (!kintoClient) {
-    let config = {
-      remote:`http://localhost:${server.identity.primaryPort}/v1/`,
-      headers: {Authorization: "Basic " + btoa("user:pass")},
-      adapter: IceCatAdapter
-    };
-    kintoClient = new Kinto(config);
-  }
-  return kintoClient.collection("test_collection");
+function do_get_kinto_sqliteHandle() {
+  return IceCatAdapter.openConnection({path: kintoFilename});
 }
 
-function* clear_collection() {
-  const collection = do_get_kinto_collection();
+function do_get_kinto_collection(sqliteHandle, collection = "test_collection") {
+  let config = {
+    remote: `http://localhost:${server.identity.primaryPort}/v1/`,
+    headers: {Authorization: "Basic " + btoa("user:pass")},
+    adapter: IceCatAdapter,
+    adapterOptions: {sqliteHandle},
+  };
+  return new Kinto(config).collection(collection);
+}
+
+async function clear_collection() {
+  let sqliteHandle;
   try {
-    yield collection.db.open();
-    yield collection.clear();
+    sqliteHandle = await do_get_kinto_sqliteHandle();
+    const collection = do_get_kinto_collection(sqliteHandle);
+    await collection.clear();
   } finally {
-    yield collection.db.close();
+    await sqliteHandle.close();
   }
 }
 
 // test some operations on a local collection
-add_task(function* test_kinto_add_get() {
-  const collection = do_get_kinto_collection();
+add_task(async function test_kinto_add_get() {
+  let sqliteHandle;
   try {
-    yield collection.db.open();
+    sqliteHandle = await do_get_kinto_sqliteHandle();
+    const collection = do_get_kinto_collection(sqliteHandle);
 
     let newRecord = { foo: "bar" };
     // check a record is created
-    let createResult = yield collection.create(newRecord);
-    do_check_eq(createResult.data.foo, newRecord.foo);
+    let createResult = await collection.create(newRecord);
+    Assert.equal(createResult.data.foo, newRecord.foo);
     // check getting the record gets the same info
-    let getResult = yield collection.get(createResult.data.id);
+    let getResult = await collection.get(createResult.data.id);
     deepEqual(createResult.data, getResult.data);
     // check what happens if we create the same item again (it should throw
     // since you can't create with id)
     try {
-      yield collection.create(createResult.data);
+      await collection.create(createResult.data);
       do_throw("Creation of a record with an id should fail");
     } catch (err) { }
     // try a few creates without waiting for the first few to resolve
@@ -62,23 +63,22 @@ add_task(function* test_kinto_add_get() {
     promises.push(collection.create(newRecord));
     promises.push(collection.create(newRecord));
     promises.push(collection.create(newRecord));
-    yield collection.create(newRecord);
-    yield Promise.all(promises);
+    await collection.create(newRecord);
+    await Promise.all(promises);
   } finally {
-    yield collection.db.close();
+    await sqliteHandle.close();
   }
 });
 
 add_task(clear_collection);
 
 // test some operations on multiple connections
-add_task(function* test_kinto_add_get() {
-  const collection1 = do_get_kinto_collection();
-  const collection2 = kintoClient.collection("test_collection_2");
-
+add_task(async function test_kinto_add_get() {
+  let sqliteHandle;
   try {
-    yield collection1.db.open();
-    yield collection2.db.open();
+    sqliteHandle = await do_get_kinto_sqliteHandle();
+    const collection1 = do_get_kinto_collection(sqliteHandle);
+    const collection2 = do_get_kinto_collection(sqliteHandle, "test_collection_2");
 
     let newRecord = { foo: "bar" };
 
@@ -91,107 +91,110 @@ add_task(function* test_kinto_add_get() {
     }
 
     // ensure subsequent operations still work
-    yield Promise.all([collection1.create(newRecord),
+    await Promise.all([collection1.create(newRecord),
                        collection2.create(newRecord)]);
-    yield Promise.all(promises);
+    await Promise.all(promises);
   } finally {
-    yield collection1.db.close();
-    yield collection2.db.close();
+    await sqliteHandle.close();
   }
 });
 
 add_task(clear_collection);
 
-add_task(function* test_kinto_update() {
-  const collection = do_get_kinto_collection();
+add_task(async function test_kinto_update() {
+  let sqliteHandle;
   try {
-    yield collection.db.open();
+    sqliteHandle = await do_get_kinto_sqliteHandle();
+    const collection = do_get_kinto_collection(sqliteHandle);
     const newRecord = { foo: "bar" };
     // check a record is created
-    let createResult = yield collection.create(newRecord);
-    do_check_eq(createResult.data.foo, newRecord.foo);
-    do_check_eq(createResult.data._status, "created");
+    let createResult = await collection.create(newRecord);
+    Assert.equal(createResult.data.foo, newRecord.foo);
+    Assert.equal(createResult.data._status, "created");
     // check we can update this OK
     let copiedRecord = Object.assign(createResult.data, {});
     deepEqual(createResult.data, copiedRecord);
     copiedRecord.foo = "wibble";
-    let updateResult = yield collection.update(copiedRecord);
+    let updateResult = await collection.update(copiedRecord);
     // check the field was updated
-    do_check_eq(updateResult.data.foo, copiedRecord.foo);
+    Assert.equal(updateResult.data.foo, copiedRecord.foo);
     // check the status is still "created", since we haven't synced
     // the record
-    do_check_eq(updateResult.data._status, "created");
+    Assert.equal(updateResult.data._status, "created");
   } finally {
-    yield collection.db.close();
+    await sqliteHandle.close();
   }
 });
 
 add_task(clear_collection);
 
-add_task(function* test_kinto_clear() {
-  const collection = do_get_kinto_collection();
+add_task(async function test_kinto_clear() {
+  let sqliteHandle;
   try {
-    yield collection.db.open();
+    sqliteHandle = await do_get_kinto_sqliteHandle();
+    const collection = do_get_kinto_collection(sqliteHandle);
 
     // create an expected number of records
     const expected = 10;
     const newRecord = { foo: "bar" };
     for (let i = 0; i < expected; i++) {
-      yield collection.create(newRecord);
+      await collection.create(newRecord);
     }
     // check the collection contains the correct number
-    let list = yield collection.list();
-    do_check_eq(list.data.length, expected);
+    let list = await collection.list();
+    Assert.equal(list.data.length, expected);
     // clear the collection and check again - should be 0
-    yield collection.clear();
-    list = yield collection.list();
-    do_check_eq(list.data.length, 0);
+    await collection.clear();
+    list = await collection.list();
+    Assert.equal(list.data.length, 0);
   } finally {
-    yield collection.db.close();
+    await sqliteHandle.close();
   }
 });
 
 add_task(clear_collection);
 
-add_task(function* test_kinto_delete(){
-  const collection = do_get_kinto_collection();
+add_task(async function test_kinto_delete() {
+  let sqliteHandle;
   try {
-    yield collection.db.open();
+    sqliteHandle = await do_get_kinto_sqliteHandle();
+    const collection = do_get_kinto_collection(sqliteHandle);
     const newRecord = { foo: "bar" };
     // check a record is created
-    let createResult = yield collection.create(newRecord);
-    do_check_eq(createResult.data.foo, newRecord.foo);
+    let createResult = await collection.create(newRecord);
+    Assert.equal(createResult.data.foo, newRecord.foo);
     // check getting the record gets the same info
-    let getResult = yield collection.get(createResult.data.id);
+    let getResult = await collection.get(createResult.data.id);
     deepEqual(createResult.data, getResult.data);
     // delete that record
-    let deleteResult = yield collection.delete(createResult.data.id);
+    let deleteResult = await collection.delete(createResult.data.id);
     // check the ID is set on the result
-    do_check_eq(getResult.data.id, deleteResult.data.id);
+    Assert.equal(getResult.data.id, deleteResult.data.id);
     // and check that get no longer returns the record
     try {
-      getResult = yield collection.get(createResult.data.id);
+      getResult = await collection.get(createResult.data.id);
       do_throw("there should not be a result");
     } catch (e) { }
   } finally {
-    yield collection.db.close();
+    await sqliteHandle.close();
   }
 });
 
-add_task(function* test_kinto_list(){
-  const collection = do_get_kinto_collection();
+add_task(async function test_kinto_list() {
+  let sqliteHandle;
   try {
-    yield collection.db.open();
+    sqliteHandle = await do_get_kinto_sqliteHandle();
+    const collection = do_get_kinto_collection(sqliteHandle);
     const expected = 10;
     const created = [];
     for (let i = 0; i < expected; i++) {
       let newRecord = { foo: "test " + i };
-      let createResult = yield collection.create(newRecord);
+      let createResult = await collection.create(newRecord);
       created.push(createResult.data);
     }
     // check the collection contains the correct number
-    let list = yield collection.list();
-    do_check_eq(list.data.length, expected);
+    let list = await collection.list();
+    Assert.equal(list.data.length, expected);
 
     // check that all created records exist in the retrieved list
     for (let createdRecord of created) {
@@ -202,73 +205,77 @@ add_task(function* test_kinto_list(){
           found = true;
         }
       }
-      do_check_true(found);
+      Assert.ok(found);
     }
   } finally {
-    yield collection.db.close();
+    await sqliteHandle.close();
   }
 });
 
 add_task(clear_collection);
 
-add_task(function* test_loadDump_ignores_already_imported_records(){
-  const collection = do_get_kinto_collection();
+add_task(async function test_loadDump_ignores_already_imported_records() {
+  let sqliteHandle;
   try {
-    yield collection.db.open();
+    sqliteHandle = await do_get_kinto_sqliteHandle();
+    const collection = do_get_kinto_collection(sqliteHandle);
     const record = {id: "41b71c13-17e9-4ee3-9268-6a41abf9730f", title: "foo", last_modified: 1457896541};
-    yield collection.loadDump([record]);
-    let impactedRecords = yield collection.loadDump([record]);
-    do_check_eq(impactedRecords.length, 0);
+    await collection.loadDump([record]);
+    let impactedRecords = await collection.loadDump([record]);
+    Assert.equal(impactedRecords.length, 0);
   } finally {
-    yield collection.db.close();
+    await sqliteHandle.close();
   }
 });
 
 add_task(clear_collection);
 
-add_task(function* test_loadDump_should_overwrite_old_records(){
-  const collection = do_get_kinto_collection();
+add_task(async function test_loadDump_should_overwrite_old_records() {
+  let sqliteHandle;
   try {
-    yield collection.db.open();
+    sqliteHandle = await do_get_kinto_sqliteHandle();
+    const collection = do_get_kinto_collection(sqliteHandle);
     const record = {id: "41b71c13-17e9-4ee3-9268-6a41abf9730f", title: "foo", last_modified: 1457896541};
-    yield collection.loadDump([record]);
+    await collection.loadDump([record]);
     const updated = Object.assign({}, record, {last_modified: 1457896543});
-    let impactedRecords = yield collection.loadDump([updated]);
-    do_check_eq(impactedRecords.length, 1);
+    let impactedRecords = await collection.loadDump([updated]);
+    Assert.equal(impactedRecords.length, 1);
   } finally {
-    yield collection.db.close();
+    await sqliteHandle.close();
   }
 });
 
 add_task(clear_collection);
 
-add_task(function* test_loadDump_should_not_overwrite_unsynced_records(){
-  const collection = do_get_kinto_collection();
+add_task(async function test_loadDump_should_not_overwrite_unsynced_records() {
+  let sqliteHandle;
   try {
-    yield collection.db.open();
+    sqliteHandle = await do_get_kinto_sqliteHandle();
+    const collection = do_get_kinto_collection(sqliteHandle);
     const recordId = "41b71c13-17e9-4ee3-9268-6a41abf9730f";
-    yield collection.create({id: recordId, title: "foo"}, {useRecordId: true});
+    await collection.create({id: recordId, title: "foo"}, {useRecordId: true});
     const record = {id: recordId, title: "bar", last_modified: 1457896541};
-    let impactedRecords = yield collection.loadDump([record]);
-    do_check_eq(impactedRecords.length, 0);
+    let impactedRecords = await collection.loadDump([record]);
+    Assert.equal(impactedRecords.length, 0);
   } finally {
-    yield collection.db.close();
+    await sqliteHandle.close();
   }
 });
 
 add_task(clear_collection);
 
-add_task(function* test_loadDump_should_not_overwrite_records_without_last_modified(){
-  const collection = do_get_kinto_collection();
+add_task(async function test_loadDump_should_not_overwrite_records_without_last_modified() {
+  let sqliteHandle;
   try {
-    yield collection.db.open();
+    sqliteHandle = await do_get_kinto_sqliteHandle();
+    const collection = do_get_kinto_collection(sqliteHandle);
     const recordId = "41b71c13-17e9-4ee3-9268-6a41abf9730f";
-    yield collection.create({id: recordId, title: "foo"}, {synced: true});
+    await collection.create({id: recordId, title: "foo"}, {synced: true});
     const record = {id: recordId, title: "bar", last_modified: 1457896541};
-    let impactedRecords = yield collection.loadDump([record]);
-    do_check_eq(impactedRecords.length, 0);
+    let impactedRecords = await collection.loadDump([record]);
+    Assert.equal(impactedRecords.length, 0);
   } finally {
-    yield collection.db.close();
+    await sqliteHandle.close();
   }
 });
 
@@ -277,11 +284,11 @@ add_task(clear_collection);
 // Now do some sanity checks against a server - we're not looking to test
 // core kinto.js functionality here (there is excellent test coverage in
 // kinto.js), more making sure things are basically working as expected.
-add_task(function* test_kinto_sync(){
+add_task(async function test_kinto_sync() {
   const configPath = "/v1/";
   const recordsPath = "/v1/buckets/default/collections/test_collection/records";
   // register a handler
-  function handleResponse (request, response) {
+  function handleResponse(request, response) {
     try {
       const sampled = getSampleResponse(request, server.identity.primaryPort);
       if (!sampled) {
@@ -292,7 +299,7 @@ add_task(function* test_kinto_sync(){
                              sampled.status.statusText);
       // send the headers
       for (let headerLine of sampled.sampleHeaders) {
-        let headerElements = headerLine.split(':');
+        let headerElements = headerLine.split(":");
         response.setHeader(headerElements[0], headerElements[1].trimLeft());
       }
       response.setHeader("Date", (new Date()).toUTCString());
@@ -306,33 +313,34 @@ add_task(function* test_kinto_sync(){
   server.registerPathHandler(recordsPath, handleResponse);
 
   // create an empty collection, sync to populate
-  const collection = do_get_kinto_collection();
+  let sqliteHandle;
   try {
     let result;
+    sqliteHandle = await do_get_kinto_sqliteHandle();
+    const collection = do_get_kinto_collection(sqliteHandle);
 
-    yield collection.db.open();
-    result = yield collection.sync();
-    do_check_true(result.ok);
+    result = await collection.sync();
+    Assert.ok(result.ok);
 
     // our test data has a single record; it should be in the local collection
-    let list = yield collection.list();
-    do_check_eq(list.data.length, 1);
+    let list = await collection.list();
+    Assert.equal(list.data.length, 1);
 
     // now sync again; we should now have 2 records
-    result = yield collection.sync();
-    do_check_true(result.ok);
-    list = yield collection.list();
-    do_check_eq(list.data.length, 2);
+    result = await collection.sync();
+    Assert.ok(result.ok);
+    list = await collection.list();
+    Assert.equal(list.data.length, 2);
 
     // sync again; the second records should have been modified
     const before = list.data[0].title;
-    result = yield collection.sync();
-    do_check_true(result.ok);
-    list = yield collection.list();
+    result = await collection.sync();
+    Assert.ok(result.ok);
+    list = await collection.list();
     const after = list.data[0].title;
-    do_check_neq(before, after);
+    Assert.notEqual(before, after);
   } finally {
-    yield collection.db.close();
+    await sqliteHandle.close();
   }
 });
 
@@ -343,7 +351,7 @@ function run_test() {
 
   run_next_test();
 
-  do_register_cleanup(function() {
+  registerCleanupFunction(function() {
     server.stop(function() { });
   });
 }
@@ -370,7 +378,16 @@ function getSampleResponse(req, port) {
         "Server: waitress"
       ],
       "status": {status: 200, statusText: "OK"},
-      "responseBody": JSON.stringify({"settings":{"batch_max_requests":25}, "url":`http://localhost:${port}/v1/`, "documentation":"https://kinto.readthedocs.org/", "version":"1.5.1", "commit":"cbc6f58", "hello":"kinto"})
+      "responseBody": JSON.stringify({
+        "settings": {
+          "batch_max_requests": 25
+        },
+        "url": `http://localhost:${port}/v1/`,
+        "documentation": "https://kinto.readthedocs.org/",
+        "version": "1.5.1",
+        "commit": "cbc6f58",
+        "hello": "kinto"
+      })
     },
     "GET:/v1/buckets/default/collections/test_collection/records?_sort=-last_modified": {
       "sampleHeaders": [
@@ -381,7 +398,14 @@ function getSampleResponse(req, port) {
         "Etag: \"1445606341071\""
       ],
       "status": {status: 200, statusText: "OK"},
-      "responseBody": JSON.stringify({"data":[{"last_modified":1445606341071, "done":false, "id":"68db8313-686e-4fff-835e-07d78ad6f2af", "title":"New test"}]})
+      "responseBody": JSON.stringify({
+        "data": [{
+          "last_modified": 1445606341071,
+          "done": false,
+          "id": "68db8313-686e-4fff-835e-07d78ad6f2af",
+          "title": "New test"
+        }]
+      })
     },
     "GET:/v1/buckets/default/collections/test_collection/records?_sort=-last_modified&_since=1445606341071": {
       "sampleHeaders": [
@@ -392,7 +416,14 @@ function getSampleResponse(req, port) {
         "Etag: \"1445607941223\""
       ],
       "status": {status: 200, statusText: "OK"},
-      "responseBody": JSON.stringify({"data":[{"last_modified":1445607941223, "done":false, "id":"901967b0-f729-4b30-8d8d-499cba7f4b1d", "title":"Another new test"}]})
+      "responseBody": JSON.stringify({
+        "data": [{
+          "last_modified": 1445607941223,
+          "done": false,
+          "id": "901967b0-f729-4b30-8d8d-499cba7f4b1d",
+          "title": "Another new test"
+        }]
+      })
     },
     "GET:/v1/buckets/default/collections/test_collection/records?_sort=-last_modified&_since=1445607941223": {
       "sampleHeaders": [
@@ -403,7 +434,14 @@ function getSampleResponse(req, port) {
         "Etag: \"1445607541265\""
       ],
       "status": {status: 200, statusText: "OK"},
-      "responseBody": JSON.stringify({"data":[{"last_modified":1445607541265, "done":false, "id":"901967b0-f729-4b30-8d8d-499cba7f4b1d", "title":"Modified title"}]})
+      "responseBody": JSON.stringify({
+        "data": [{
+          "last_modified": 1445607541265,
+          "done": false,
+          "id": "901967b0-f729-4b30-8d8d-499cba7f4b1d",
+          "title": "Modified title"
+        }]
+      })
     }
   };
   return responses[`${req.method}:${req.path}?${req.queryString}`] ||
