@@ -11,8 +11,10 @@
 #include "nsPoint.h"
 #include "nsRegion.h"
 #include "nsCRT.h"
+#include "nsCOMPtr.h"
 #include "nsWidgetInitData.h" // for nsWindowType
 #include "nsIWidgetListener.h"
+#include "Units.h"
 #include "mozilla/EventForwards.h"
 
 class nsViewManager;
@@ -21,7 +23,7 @@ class nsIFrame;
 
 // Enumerated type to indicate the visibility of a layer.
 // hide - the layer is not shown.
-// show - the layer is shown irrespective of the visibility of 
+// show - the layer is shown irrespective of the visibility of
 //        the layer's parent.
 enum nsViewVisibility {
   nsViewVisibility_kHide = 0,
@@ -56,7 +58,12 @@ class nsView final : public nsIWidgetListener
 public:
   friend class nsViewManager;
 
-  NS_DECL_AND_IMPL_ZEROING_OPERATOR_NEW
+  typedef mozilla::LayoutDeviceIntRect LayoutDeviceIntRect;
+  typedef mozilla::LayoutDeviceIntRegion LayoutDeviceIntRegion;
+
+  void operator delete(void* ptr) {
+    ::operator delete(ptr);
+  }
 
   /**
    * Get the view manager which "owns" the view.
@@ -129,7 +136,7 @@ public:
    *
    * If aOther is null, this will return the offset of |this| from the
    * root of the viewmanager tree.
-   * 
+   *
    * This function is fastest when aOther is an ancestor of |this|.
    *
    * NOTE: this actually returns the offset from aOther to |this|, but
@@ -280,15 +287,25 @@ public:
    */
   nsIWidget* GetWidget() const { return mWindow; }
 
+  /**
+   * The widget which we have attached a listener to can also have a "previous"
+   * listener set on it. This is to keep track of the last nsView when navigating
+   * to a new one so that we can continue to paint that if the new one isn't ready
+   * yet.
+   */
+  void SetPreviousWidget(nsIWidget* aWidget) { mPreviousWindow = aWidget; }
+
   /**
    * Returns true if the view has a widget associated with it.
    */
   bool HasWidget() const { return mWindow != nullptr; }
-  
-  void SetForcedRepaint(bool aForceRepaint) { 
-    mForcedRepaint = aForceRepaint; 
+
+  void SetForcedRepaint(bool aForceRepaint) {
+    mForcedRepaint = aForceRepaint;
   }
 
+  void SetNeedsWindowPropertiesSync();
+
   /**
    * Make aWidget direct its events to this view.
    * The caller must call DetachWidgetEventHandler before this view
@@ -315,7 +332,7 @@ public:
    */
   bool IsRoot() const;
 
-  nsIntRect CalcWidgetBounds(nsWindowType aType);
+  LayoutDeviceIntRect CalcWidgetBounds(nsWindowType aType);
 
   // This is an app unit offset to add when converting view coordinates to
   // widget coordinates.  It is the offset in view coordinates from widget
@@ -368,9 +385,12 @@ public:
   virtual bool WindowResized(nsIWidget* aWidget, int32_t aWidth, int32_t aHeight) override;
   virtual bool RequestWindowClose(nsIWidget* aWidget) override;
   virtual void WillPaintWindow(nsIWidget* aWidget) override;
-  virtual bool PaintWindow(nsIWidget* aWidget, nsIntRegion aRegion) override;
+  virtual bool PaintWindow(nsIWidget* aWidget,
+                           LayoutDeviceIntRegion aRegion) override;
   virtual void DidPaintWindow() override;
-  virtual void DidCompositeWindow() override;
+  virtual void DidCompositeWindow(uint64_t aTransactionId,
+                                  const mozilla::TimeStamp& aCompositeStart,
+                                  const mozilla::TimeStamp& aCompositeEnd) override;
   virtual void RequestRepaint() override;
   virtual nsEventStatus HandleEvent(mozilla::WidgetGUIEvent* aEvent,
                                     bool aUseAttachedEvents) override;
@@ -380,6 +400,8 @@ public:
   nsPoint GetOffsetTo(const nsView* aOther, const int32_t aAPD) const;
   nsIWidget* GetNearestWidget(nsPoint* aOffset, const int32_t aAPD) const;
 
+  bool IsPrimaryFramePaintSuppressed();
+
 private:
   explicit nsView(nsViewManager* aViewManager = nullptr,
                   nsViewVisibility aVisibility = nsViewVisibility_kShow);
@@ -438,15 +460,13 @@ private:
 
   void NotifyEffectiveVisibilityChanged(bool aEffectivelyVisible);
 
-  // Update the cached RootViewManager for all view manager descendents,
-  // If the hierarchy is being removed, aViewManagerParent points to the view
-  // manager for the hierarchy's old parent, and will have its mouse grab
-  // released if it points to any view in this view hierarchy.
-  void InvalidateHierarchy(nsViewManager *aViewManagerParent);
+  // Update the cached RootViewManager for all view manager descendents.
+  void InvalidateHierarchy();
 
   nsViewManager    *mViewManager;
   nsView           *mParent;
-  nsIWidget        *mWindow;
+  nsCOMPtr<nsIWidget> mWindow;
+  nsCOMPtr<nsIWidget> mPreviousWindow;
   nsView           *mNextSibling;
   nsView           *mFirstChild;
   nsIFrame         *mFrame;
@@ -462,6 +482,7 @@ private:
   uint32_t          mVFlags;
   bool              mWidgetIsTopLevel;
   bool              mForcedRepaint;
+  bool              mNeedsWindowPropertiesSync;
 };
 
 #endif
